/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 *
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2017 by TOPPERS PROJECT Educational Working Group.
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id: adxl345.h 59 2017-05-30 18:46:28Z roi $
 */

/*
 *  ADXL345テストプログラムのヘッダファイル
 */

/*
 *  ターゲット依存の定義
 */
#include "target_test.h"

/*
 *  各タスクの優先度の定義
 */

#define MAIN_PRIORITY	5		/* メインタスクの優先度 */
								/* HIGH_PRIORITYより高くすること */

#define SERVER_PRIORITY	4		/* サーバタスクの優先度 */

#define HIGH_PRIORITY	9		/* 並行に実行されるタスクの優先度 */
#define MID_PRIORITY	10
#define LOW_PRIORITY	11

/*
 *  SERVERタスクへの要求のためのDTQのバッファ数
 */
#define NUM_SERVER_DTQ		10

/*
 *  ターゲットに依存する可能性のある定数の定義
 */

#ifndef TASK_PORTID_PRC1
#define	TASK_PORTID_PRC1		1	/* プロセッサ1で文字入力するシリアルポートID */
#endif /* TASK_PORTID_PRC1 */

#ifndef TASK_PORTID_PRC2
#define	TASK_PORTID_PRC2		2	/* プロセッサ2で文字入力するシリアルポートID */
#endif /* TASK_PORTID_PRC2 */

#ifndef TASK_PORTID_PRC3
#define	TASK_PORTID_PRC3		3	/* プロセッサ3で文字入力するシリアルポートID */
#endif /* TASK_PORTID_PRC3 */

#ifndef TASK_PORTID_PRC4
#define	TASK_PORTID_PRC4		4	/* プロセッサ4で文字入力するシリアルポートID */
#endif /* TASK_PORTID_PRC4 */

#ifndef TASK_PORTID_G_SYSLOG
#define	TASK_PORTID_G_SYSLOG	1	/* G_SYSLOG時に文字入力するシリアルポートID */
#endif /* TASK_PORTID_G_SYSLOG */

#ifndef STACK_SIZE
#define	STACK_SIZE		4096		/* タスクのスタックサイズ */
#endif /* STACK_SIZE */

#ifndef LOOP_REF
#define LOOP_REF		ULONG_C(1000000)	/* 速度計測用のループ回数 */
#endif /* LOOP_REF */

#ifndef ALM_TIME
#define ALM_TIME 5000
#endif /* ALMTIM */

#ifndef CYC_TIME
#define CYC_TIME 10000
#endif /* ALMTIM */

#define INHNO_I2C1  (0x00010000 | INTNO_I2C0_IRQ)	/* 割込みハンドラ番号 */
#define INTNO_I2C1  INTNO_I2C0_IRQ	/* 割込み番号 */
#define INTPRI_I2C1 -9			/* 割込み優先度 */
#define INTATR_I2C1 0			/* 割込み属性 */

/*
 *  関数のプロトタイプ宣言
 */
#ifndef TOPPERS_MACRO_ONLY

extern void	task(intptr_t exinf);
extern void	main_task(intptr_t exinf);

extern void	tex_routine(TEXPTN texptn, intptr_t exinf);
#ifdef CPUEXC1
extern void	cpuexc_handler(void *p_excinf);
#endif /* CPUEXC1 */
extern void	cyclic_handler(intptr_t exinf);
extern void	alarm_handler(intptr_t exinf);

extern void global_inirtn(intptr_t exinf);
extern void global_terrtn(intptr_t exinf);
extern void local_inirtn(intptr_t exinf);
extern void local_terrtn(intptr_t exinf);

extern void sw_int(void);
extern void device_info_init(intptr_t exinf);


#endif /* TOPPERS_MACRO_ONLY */
