/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: sample1.c 64 2009-01-13 16:37:04Z ertl-honda $
 */

/* 
 *  サンプルプログラム(1)の本体
 *
 *  FMPカーネルの基本的な動作を確認するためのサンプルプログラム．
 *
 *  プログラムの概要:
 *
 *  ユーザインタフェースを受け持つメインタスク（タスクID: MAIN_TASK，優
 *  先度: MAIN_PRIORITY）と，3つの並行実行されるタスク（タスクID:
 *  TASK1～TASK3，初期優先度: MID_PRIORITY）で構成される．また，起動周
 *  期が2秒の周期ハンドラ（周期ハンドラID: CYCHDR1）を用いる．
 *
 *  並行実行されるタスクは，task_loop回空ループを実行する度に，タスクが
 *  実行中であることをあらわすメッセージを表示する．空ループを実行する
 *  のは，空ループなしでメッセージを出力すると，多量のメッセージが出力
 *  され，プログラムの動作が確認しずらくなるためである．また，低速なシ
 *  リアルポートを用いてメッセージを出力する場合に，すべてのメッセージ
 *  が出力できるように，メッセージの量を制限するという理由もある．
 *
 *  周期ハンドラは，三つの優先度（HIGH_PRIORITY，MID_PRIORITY，
 *  LOW_PRIORITY）のレディキューを回転させる．プログラムの起動直後は，
 *  周期ハンドラは停止状態になっている．
 *
 *  メインタスクは，シリアルI/Oポートからの文字入力を行い（文字入力を
 *  待っている間は，並行実行されるタスクが実行されている），入力された
 *  文字に対応した処理を実行する．入力された文字と処理の関係は次の通り．
 *  Control-Cまたは'Q'が入力されると，プログラムを終了する．
 *
 *  '1' : 対象タスクをTASK1に切り換える（初期設定）．
 *  '2' : 対象タスクをTASK2に切り換える．
 *  '3' : 対象タスクをTASK3に切り換える．
 *  '4' : クラス1に所属するタスクを指定する．
 *        対象周期ハンドラをCYCHDR1に切り替える．
 *        対象アラームハンドラをALMHDR1に切り替える．
 *  '5' : クラス2に所属するタスクを指定する．
 *        対象周期ハンドラをCYCHDR2に切り替える．
 *        対象アラームハンドラをALMHDR2に切り替える．
 *  '6' : クラス3に所属するタスクを指定する．
 *        対象周期ハンドラをCYCHDR3に切り替える．
 *        対象アラームハンドラをALMHDR3に切り替える．
 *  '7' : クラス4に所属するタスクを指定する．
 *        対象周期ハンドラをCYCHDR4に切り替える．
 *        対象アラームハンドラをALMHDR4に切り替える．
 *  '8' : プロセッサ1を指定する．
 *  '9' : プロセッサ2を指定する．
 *  '0' : プロセッサ3を指定する．
 *  '-' : プロセッサ4を指定する．
 *  'a' : 対象タスクをact_tskにより起動する．
 *  'A' : 対象タスクに対する起動要求をcan_actによりキャンセルする．
 *  'e' : 対象タスクにext_tskを呼び出させ，終了させる．
 *  't' : 対象タスクをter_tskにより強制終了する．
 *  '>' : 対象タスクの優先度をHIGH_PRIORITYにする．
 *  '=' : 対象タスクの優先度をMID_PRIORITYにする．
 *  '<' : 対象タスクの優先度をLOW_PRIORITYにする．
 *  'G' : 対象タスクの優先度をget_priで読み出す．
 *  's' : 対象タスクにslp_tskを呼び出させ，起床待ちにさせる．
 *  'S' : 対象タスクにtslp_tsk(10秒)を呼び出させ，起床待ちにさせる．
 *  'w' : 対象タスクをwup_tskにより起床する．
 *  'W' : 対象タスクに対する起床要求をcan_wupによりキャンセルする．
 *  'l' : 対象タスクをrel_waiにより強制的に待ち解除にする．
 *  'u' : 対象タスクをsus_tskにより強制待ち状態にする．
 *  'm' : 対象タスクの強制待ち状態をrsm_tskにより解除する．
 *  'd' : 対象タスクにdly_tsk(10秒)を呼び出させ，時間経過待ちにさせる．
 *  'x' : 対象タスクに例外パターン0x0001の例外処理を要求する．
 *  'X' : 対象タスクに例外パターン0x0002の例外処理を要求する．
 *  'y' : 対象タスクにdis_texを呼び出させ，タスク例外を禁止する．
 *  'Y' : 対象タスクにena_texを呼び出させ，タスク例外を許可する．
 *  'r' : 3つの優先度（HIGH_PRIORITY，MID_PRIORITY，LOW_PRIORITY）のレ
 *        ディキューを回転させる．
 *  'c' : 周期ハンドラを動作開始させる．
 *  'C' : 周期ハンドラを動作停止させる．
 *  'I' : 周期ハンドラの割付けプロセッサを変更する． 
 *  'b' : アラームハンドラを5秒後に起動するよう動作開始させる．
 *  'B' : アラームハンドラを動作停止させる．
 *  'E' : アラームハンドラの割付けプロセッサを変更する．   
 *  'z' : 対象タスクにCPU例外を発生させる（タスクを終了させる）．
 *  'Z' : 対象タスクにCPUロック状態でCPU例外を発生させる（プログラムを
 *        終了する）．
 *  'i' : 対象タスクをmig_tskにより割付けプロセッサを変更する． 
 *  'g' : 対象タスクにmig_tskを呼び出させ，割付けプロセッサを変更する． 
 *  'f' : 対象タスクをmact_tskにより起動する．  
 *  'p' : 対象タスクにget_pidを呼び出させ，プロセッサ番号を表示させる．
 *  'V' : get_utmで性能評価用システム時刻を2回読む．
 *  'v' : 発行したシステムコールを表示する（デフォルト）．
 *  'q' : 発行したシステムコールを表示しない．
 */

#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include <sil.h>
#include <string.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "device.h"
#include "qspi.h"
#include "qspitest.h"


/*
 *  サービスコールのエラーのログ出力
 */
Inline void
svc_perror(const char *file, int_t line, const char *expr, ER ercd)
{
	if (ercd < 0) {
		t_perror(LOG_ERROR, file, line, expr, ercd);
	}
}

#define	SVC_PERROR(expr)	svc_perror(__FILE__, __LINE__, #expr, (expr))

/*
 *  シリアルポート番号のテーブル
 */
uint_t const serial_port[TNUM_PRCID] = {
#ifndef G_SYSLOG
	TASK_PORTID_PRC1,
#else /* G_SYSLOG */
	TASK_PORTID_G_SYSLOG,
#endif /* G_SYSLOG */
    
#if TNUM_PRCID >= 2
    TASK_PORTID_PRC2,
#endif /* TNUM_PRCID >= 2 */
};

uint32_t buffer[1028];

/*
 *  タスクのマイグレーション先のプロセッサの指定
 */
ID  tsk_mig_prc;


#define ALT_QSPI_SUPPORT_MICRON_N25Q128         1
#define ALT_QSPI_SUPPORT_MICRON_N25Q512A        1
#define ALT_QSPI_SUPPORT_MICRON_N25Q00AA        0

#define ALT_QSPI_ERASE_CHIP_SIZE 	0xffffffff
#define ALT_QSPI_TIMEOUT_INFINITE (0xffffffff)

/* static functions */
#define ALT_QSPI_CFG_SELCLKPHASE_E_ACT      0x0
#define ALT_QSPI_CFG_SELCLKPOL_E_HIGH   0x0

/****************************************************************************
*   Device Specific Code
****************************************************************************/


#if (ALT_QSPI_SUPPORT_MICRON_N25Q128==1)
#define ALT_QSPI_DEVICE_ID_MICRON_N25Q128         0x18BA

static ER alt_qspi_N25Q128_enable(void *h);
QSPI_Init_t qspi_config_MICRON_N25Q128 =
{
	QSPI_STIG_RDID_JEDECID_MICRON,	/* manuf_id */
    ALT_QSPI_DEVICE_ID_MICRON_N25Q128,	/* type_capacity */
    ALT_QSPI_CFG_SELCLKPHASE_E_ACT, /* .clk_phase */
    ALT_QSPI_CFG_SELCLKPOL_E_HIGH, /* .clk_pol */
    50,   /* .max_freq_mhz */
    100,  /* .tshsl_ns */
    30,   /* .tsd2d_ns */
    30,   /* .tchsh_ns */
    30,   /* .tslch_ns */
    256,  /* .page_size */
    2,    /* .addr_size */
    QSPI_MODE_SINGLE, /* .inst_type */
    QSPI_STIG_OPCODE_FASTREAD_QUAD_IO,	/* .read_op_code */
    QSPI_MODE_QUAD,	/* .read_addr_xfer_type */
    QSPI_MODE_QUAD,	/* .read_data_xfer_type */
    10,   /* .read_dummy_cycles */
    QSPI_STIG_OPCODE_4BYTE_WRITE,	/* .write_op_code */
    QSPI_MODE_QUAD, /* .write_addr_xfer_type */
    QSPI_MODE_QUAD, /* .write_data_xfer_type */
    0,    /* .write_dummy_cycles */
    2,    /* .erase_count */
    {64*1024, 16*1024*1024}, /* .erase_sizes */
    {QSPI_STIG_OPCODE_SEC_ERASE,
     QSPI_STIG_OPCODE_BULK_ERASE}, /* .erase_cmds */
    0,    /* .erase_sector_idx */
    true, /* .support_chip_erase */
    0x01000000,    /* .device_size - 128Mb */
    false,/* .micron_multi_die */
    0,    /* die-size */ 
    alt_qspi_N25Q128_enable, /* .init_func */
    qspi_sr_wait_write, /* .wait_func */
    "MICRON_N25Q128"
};
#endif

#if (ALT_QSPI_SUPPORT_MICRON_N25Q00AA==1)
#define ALT_QSPI_DEVICE_ID_MICRON_N25Q00AA        0x21BA
static ER alt_qspi_N25Q512A_N25Q00AA_enable(void *h);
static ER alt_qspi_N25Q512A_N25Q00AA_wait_write(void *h, uint32_t timeout);
QSPI_Init_t qspi_config_MICRON_N25Q00AA =
{
	QSPI_STIG_RDID_JEDECID_MICRON,	/* manuf_id */
    ALT_QSPI_DEVICE_ID_MICRON_N25Q00AA,	/* type_capacity */
    ALT_QSPI_CFG_SELCLKPHASE_E_ACT, /* .clk_phase */
    ALT_QSPI_CFG_SELCLKPOL_E_HIGH,   /* .clk_pol */
    56,   /* .max_freq_mhz */
    50,  /* .tshsl_ns */
    0,   /* .tsd2d_ns */
    10,   /* .tchsh_ns */
    10,   /* .tslch_ns */
    256,  /* .page_size */
    3,    /* .addr_size */
    QSPI_MODE_SINGLE, /* .inst_type */
    QSPI_STIG_OPCODE_FASTREAD_QUAD_IO, /* .read_op_code */
    QSPI_MODE_QUAD, /* .read_addr_xfer_type */
    QSPI_MODE_QUAD, /* .read_data_xfer_type */
    10,   /* .read_dummy_cycles */
    QSPI_STIG_OPCODE_PP,	/* .write_op_code */
    QSPI_MODE_SINGLE,	/* .write_addr_xfer_type */
    QSPI_MODE_SINGLE,	/* .write_data_xfer_type */
    0,    /* .write_dummy_cycles */
    3,    /* .erase_count */
    {4*1024, 64*1024, 128*1024*1024}, /* .erase_sizes */
    {QSPI_STIG_OPCODE_SUBSEC_ERASE,
     QSPI_STIG_OPCODE_SEC_ERASE,
     QSPI_STIG_OPCODE_DIE_ERASE}, /* .erase_cmds */
    1,    /* .erase_sector_idx */
    false,/* .support_chip_erase */
    0x08000000,    /* .device_size - 1Gb */
    true, /* .micron_multi_die */
    32*1024*1024, /* .die_size */
    alt_qspi_N25Q512A_N25Q00AA_enable, /* .init_func */
    alt_qspi_N25Q512A_N25Q00AA_wait_write, /* .wait_func */
    "MICRON_N25Q00AA"
};
#endif

#if (ALT_QSPI_SUPPORT_MICRON_N25Q512A==1)
#define ALT_QSPI_DEVICE_ID_MICRON_N25Q512A        0x20BA
static ER alt_qspi_N25Q512A_N25Q00AA_enable(void *h);
static ER alt_qspi_N25Q512A_N25Q00AA_wait_write(void *h, uint32_t timeout);
QSPI_Init_t qspi_config_MICRON_N25Q512A =
{
	QSPI_STIG_RDID_JEDECID_MICRON,	/* manuf_id */
    ALT_QSPI_DEVICE_ID_MICRON_N25Q512A,	/* type_capacity */
    ALT_QSPI_CFG_SELCLKPHASE_E_ACT, /* .clk_phase */
    ALT_QSPI_CFG_SELCLKPOL_E_HIGH,   /* .clk_pol */
/**********************************************************************
    Quad SPI specs from data sheet:
     Max input data start time = ((2 + R) * Tq) - 7.52 nsec
     Min input data end time = ((2 + R) * Tq) - 1.21 nsec
     Where:
       R = read data delay
       Tq = period of QSPI base clock
   Micron flash specs from N25Q00AA data sheet:
     Micron flash max output delay = 8 nsec
  
   We can satisfy the input data end time with
     ((2 + R) * Tq) - 1.21 <= Tb   Tb = period of the baud clock
  
     (2 + R) * Tq <= Tb + 1.21
     Disregard the 1.21 nsec because Tb < Tb + 1.21
  
     (2 + R) * Tq <= Tb    The min value of Tb
  
     For baud rate divisor D = Tb / Tq
  
     R <= D - 2     Calculate read data delay from baud rate divisor
  
   We can satisfy the input data start time with
     ((2 + R) * Tq) - 7.52 >= 8 + P
     Where:
       8 = flash max output delay
       P = round trip propagation time from SOC to flash
  
     P = 2 nsec would allow up to 6 inches between SOC and flash
  
     (2 + R) * Tq >= 17.52 nsec
  
     Set min Tb = 17.52 nsec gives max baud rate  = 1/17.52 nsec = 57 MHz
     Let's use 56 MHz to provide a bit of margin 
************************************************************************/
    56,   /* .max_freq_mhz */
    50,  /* .tshsl_ns */
    0,   /* .tsd2d_ns */
    10,   /* .tchsh_ns */
    10,   /* .tslch_ns */
    256,  /* .page_size */
    3,    /* .addr_size */
    QSPI_MODE_SINGLE, /* .inst_type */
    QSPI_STIG_OPCODE_FASTREAD_QUAD_IO,	/* .read_op_code */
    QSPI_MODE_QUAD, /* .read_addr_xfer_type */
    QSPI_MODE_QUAD, /* .read_data_xfer_type */
    10,   /* .read_dummy_cycles */
    QSPI_STIG_OPCODE_PP,	/* .write_op_code */
    QSPI_MODE_SINGLE,	/* .write_addr_xfer_type */
    QSPI_MODE_SINGLE,	/* .write_data_xfer_type */
    0,    /* .write_dummy_cycles */
    3,    /* .erase_count */
    {4*1024, 64*1024, 64*1024*1024},	/* .erase_sizes */
    {QSPI_STIG_OPCODE_SUBSEC_ERASE,
     QSPI_STIG_OPCODE_SEC_ERASE,
     QSPI_STIG_OPCODE_DIE_ERASE},	/* .erase_cmds */
    1,    /* .erase_sector_idx */
    false,/* .support_chip_erase */
    0x04000000,    /* .device_size - 512Mb */
    true, /* .micron_multi_die */
    32*1024*1024, /* .die_size */
    alt_qspi_N25Q512A_N25Q00AA_enable, /* .init_func */
    alt_qspi_N25Q512A_N25Q00AA_wait_write, /* .wait_func */
    "MICRON_N25Q512A"
};
#endif

#if (ALT_QSPI_SUPPORT_MICRON_N25Q512A==1) || (ALT_QSPI_SUPPORT_MICRON_N25Q00AA==1)

static ER alt_qspi_N25Q512A_N25Q00AA_enable(void *h)
{
	QSPI_Handle_t *hqspi = h;
    ER ercd = E_OK;
    uint32_t reg;

    /* Reset the volatile memory on the N25Q */
	reg = (QSPI_STIG_OPCODE_N25Q512A_RESET_EN<<24);
	ercd = qspi_stig_cmd(hqspi, reg, ALT_QSPI_TIMEOUT_INFINITE);

    /* Reset memory */
	if(ercd == E_OK){
		reg = (QSPI_STIG_OPCODE_N25Q512A_RESET_MEM<<24);
		ercd = qspi_stig_cmd(hqspi, reg, ALT_QSPI_TIMEOUT_INFINITE);
	}

    /* Write enable */
	if(ercd == E_OK){
		reg = (QSPI_STIG_OPCODE_WREN<<24);
		ercd = qspi_stig_cmd(hqspi, reg, 10000);
	}

	/* Switch to 4 byte mode */
	if(ercd == E_OK){
		reg = (QSPI_STIG_OPCODE_ENTER_4BYTE_MODE<<24);
		ercd = qspi_stig_cmd(hqspi, reg, 10000);
	}
	return ercd;
}

static ER alt_qspi_N25Q512A_N25Q00AA_wait_write(void *h, uint32_t time_out)
{
	QSPI_Handle_t *hqspi = h;
    ER ercd = E_OK;
    uint8_t fsr = 0;
	uint32_t data;
    bool_t infinite = (time_out == ALT_QSPI_TIMEOUT_INFINITE);

    /* Wait for the status register as for the other devices */
	ercd = qspi_sr_wait_write(h, time_out);

    /* Wait for the flag status register */
    do{
        /* Read flag status register */
	    ercd = qspi_stig_rd_cmd(hqspi, QSPI_STIG_OPCODE_N25Q512A_RDFLGSR, 0, 1, &data, ALT_QSPI_TIMEOUT_INFINITE);
	    fsr = (uint8_t)data;
        if(ercd != E_OK)
			break;
		if(fsr & 0x80)
			break;
    }
    while(time_out-- || infinite);

    if(time_out == (uint32_t)-1 && !infinite)
		ercd = E_TMOUT;
	return ercd;

}
#endif

#if (ALT_QSPI_SUPPORT_MICRON_N25Q128==1)

static ER alt_qspi_N25Q128_enable(void *h)
{
	QSPI_Handle_t *hqspi = h;
    ER ercd = E_OK;
    uint32_t data;
    uint8_t * vecr = (uint8_t*)&data;
    uint32_t reg;

    /* Read VECR */
	ercd = qspi_stig_rd_cmd(hqspi, QSPI_STIG_REGISTER_N25Q128_VECR, 0, 1, &data, ALT_QSPI_TIMEOUT_INFINITE);

    /* Write enable */
    if(ercd == E_OK){
	    reg = (QSPI_STIG_OPCODE_WREN<<24);
		ercd = qspi_stig_cmd(hqspi, reg, 10000);
    }

    /* Return if Reset/Hold pad functionality is already disabled */
    if((ercd == E_OK) && !(*vecr & 0x10))
        return ercd;

    /* Disable Reset/Hold pad functionality */
	if(ercd == E_OK){
		*vecr &= ~0x10;
		ercd = qspi_stig_wr_cmd(hqspi, QSPI_STIG_OPCODE_N25Q128, 0, 1, (uint32_t*)vecr, ALT_QSPI_TIMEOUT_INFINITE);
    }

    /* Wait for completion */
    if(ercd == E_OK)
		ercd = qspi_sr_wait_write(h, ALT_QSPI_TIMEOUT_INFINITE);
	return ercd;
}
#endif

/*
 *  CPU例外ハンドラ
 */
#ifdef CPUEXC1

void
cpuexc_handler(void *p_excinf)
{
	ID		tskid;

	syslog(LOG_NOTICE, "CPU exception handler1 (p_excinf = %08p).", p_excinf);
	if (sns_ctx() != true) {
		syslog(LOG_WARNING,
					"sns_ctx() is not true in CPU exception handler.");
	}
	if (sns_dpn() != true) {
		syslog(LOG_WARNING,
					"sns_dpn() is not true in CPU exception handler.");
	}
	syslog(LOG_INFO, "sns_loc = %d sns_dsp = %d sns_tex = %d",
									sns_loc(), sns_dsp(), sns_tex());

	syslog(LOG_INFO, "xsns_dpn = %d xsns_xpn = %d",
									xsns_dpn(p_excinf), xsns_xpn(p_excinf));

    
	if (xsns_xpn(p_excinf)) {
		syslog(LOG_NOTICE, "Sample program ends with exception.");
		SVC_PERROR(ext_ker());
		assert(0);
	}

	SVC_PERROR(iget_tid(&tskid));
	SVC_PERROR(iras_tex(tskid, 0x8000U));
}

#endif /* CPUEXC1 */

static uint_t counter;

/*
 *  グローバル初期化ルーチン
 */
void
global_inirtn(intptr_t exinf)
{
	counter = 1;
}

/*
 *  グローバル終了ルーチン
 */
void
global_terrtn(intptr_t exinf)
{
	syslog(LOG_EMERG, "global_terrtn exinf = %d, counter = %d", exinf, counter++);
}

/*
 *  ローカル初期化ルーチン
 */
void
local_inirtn(intptr_t exinf)
{
	syslog(LOG_NOTICE, "local_inirtn exinf = %d, counter = %d", exinf, counter++);
}

/*
 *  ローカル終了ルーチン
 */
void
local_terrtn(intptr_t exinf)
{
	syslog(LOG_NOTICE, "local_terrtn exinf = %d, counter = %d", exinf, counter++);
}

#if 1	/* ROI DEBUG */
void sw_int(void)
{
	syslog_0(LOG_NOTICE, "## sw_int() ##");
}
#endif	/* ROI DEBUG */

void
cyclic_handler(intptr_t exinf)
{
	wdog_reset(WDOG1_PORTID);
}

void
qspi1_hps_handler(void)
{
	qspi_hps_handler(QSPI1_PORTID);
}

/*
 *  メインタスク
 */
void main_task(intptr_t exinf)
{
	QSPI_Handle_t *hqspi;
	ER status = E_OK;
	uint32_t addr;
	uint8_t  *p;
	ER_UINT	ercd;
	int i;

	tsk_mig_prc = 1;

	SVC_PERROR(syslog_msk_log(LOG_UPTO(LOG_NOTICE), LOG_UPTO(LOG_EMERG)));
	syslog(LOG_NOTICE, "Sample program starts (exinf = %d).", (int_t) exinf);

	/*
	 *  シリアルポートの初期化
	 *
	 *  システムログタスクと同じシリアルポートを使う場合など，シリアル
	 *  ポートがオープン済みの場合にはここでE_OBJエラーになるが，支障は
	 *  ない．
	 */
	ercd = serial_opn_por(((int_t)serial_port[exinf-1]));
	if (ercd < 0 && MERCD(ercd) != E_OBJ) {
		syslog(LOG_ERROR, "%s (%d) reported by `serial_opn_por'.",
									itron_strerror(ercd), SERCD(ercd));
	}
	SVC_PERROR(serial_ctl_por(serial_port[exinf-1],
							(IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));

	hqspi = qspi_init(QSPI1_PORTID, &qspi_config_MICRON_N25Q512A);
	syslog_1(LOG_NOTICE, "## qspi_init[%08x] ##", hqspi);
	if(hqspi == NULL){
		syslog_0(LOG_NOTICE, "## qspi initial error !(STOP) ##");
		slp_tsk();
	}
	hqspi->semid = QSPITRS1_SEM;

	status = qspi_erase(hqspi, 0, 64*1024);
	syslog_1(LOG_NOTICE, "## qspi_erase(%d) ##", status);
	dly_tsk(200);
	p = (uint8_t *)buffer;
	for(i = 0 ; i < 1028 ; i++)
		*p++ = i;
	status = qspi_write(hqspi, 0, buffer, 1028);
	syslog_1(LOG_NOTICE, "## qspi_write(%d) ##", status);
	dly_tsk(200);

	dly_tsk(200);

	memset(buffer, 0xaa, 2048);
	status = qspi_read(hqspi, buffer, 0, 1032);
	syslog_2(LOG_NOTICE, "## qspi_read(%d)[%08x] ##", status, buffer);

	addr = qspi_ahb_remap_address_get(hqspi);
	syslog_1(LOG_NOTICE, "## addr[%08x] ##", addr);
	addr = qspi_get_die_size(hqspi);
	syslog_1(LOG_NOTICE, "## die size[%08x] ##", addr);
	qspi_direct_enable(hqspi, 0);

	addr = qspi_get_device_size(hqspi);
	syslog_1(LOG_NOTICE, "## device size[%08x] ##", addr);
	addr = qspi_get_page_size(hqspi);
	syslog_1(LOG_NOTICE, "## page size[%08x] ##", addr);
	addr = get_smallest_sector_size(hqspi);
	syslog_1(LOG_NOTICE, "## sector size[%08x] ##", addr);

	dly_tsk(200);
	slp_tsk();

	qspi_direct_disable(hqspi);

	status = qspi_deinit(hqspi);
	syslog_1(LOG_NOTICE, "## alt_qspi_uninit(%d) ##", status);

	syslog_0(LOG_NOTICE, "## STOP ##");
	slp_tsk();

	syslog(LOG_NOTICE, "Sample program ends.");
	SVC_PERROR(ext_ker());
	assert(0);
}
