/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 *
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2017 by TOPPERS PROJECT Educational Working Group.
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id: fpgaspi.h 59 2017-08-21 11:02:16Z roi $
 */

/*
 *  CycloneV FPGA SPIデバイスドライバの外部宣言
 */

#ifndef _FPGASPI_H_
#define _FPGASPI_H_

#include "fpga.h"
#include "socfpga_cv.h"

#ifdef __cplusplus
 extern "C" {
#endif

/*
 *  FPGASPIポート定義
 */
#define FPGASPI1_PORTID         1
#define NUM_FPGASPIPORT         1

/*
 *  転送後SS信号を落とさない設定
 */
#define AVALON_SPI_COMMAND_MERGE    0x01

/*
 *  SS-TOGGLE信号を変化させない設定
 */
#define AVALON_SPI_COMMAND_TOGGLE_SS_N 0x02


#ifndef TOPPERS_MACRO_ONLY

/*
 *  拡張UARTハンドラ定義
 */
typedef struct fpgaspi_struct FPGASPI_Handle_t;
struct fpgaspi_struct {
	uint32_t    base;				/* SPI registers base address */
	uint32_t	portid;				/* SPI ポートID */
	uint8_t     *txBuff;			/* SPI 送信データ領域へのポインタ */
	uint8_t     *rxBuff;			/* SPI 受信データ領域へのポインタ */
	uint32_t    size;				/* SPI 転送サイズ */
	uint32_t    txCount;			/* SPI 送信カウンタ */
	uint32_t    rxCount;			/* SPI 受信カウンタ */
	uint32_t    flags;				/* SPI 設定フラグ */
	uint32_t    timeout;			/* SPI タイムアウト値(ms) */
	int         semid;				/* SPI 通信用セマフォ値 */
	int         semlock;			/* SPI ロックセマフォ値 */
};

#endif /* TOPPERS_MACRO_ONLY */

/*
 *  拡張UART関数のプロトタイプ宣言
 */
#ifndef TOPPERS_MACRO_ONLY

extern FPGASPI_Handle_t *fpgaspi_init(ID portid);
extern ER fpgaspi_deinit(FPGASPI_Handle_t *hspi);
extern int  fpgaspi_transrecv(FPGASPI_Handle_t *hspi, uint32_t slave_select, uint8_t *tx_buf, uint8_t *rx_buf, uint32_t len);
extern void spim_fpga_isr(intptr_t exinf);


#endif /* TOPPERS_MACRO_ONLY */

#ifdef __cplusplus
}
#endif

#endif	/* _FPGASPI_H_ */

