/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 *
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2017 by TOPPERS PROJECT Educational Working Group.
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id: fpgauart.h 59 2017-04-17 14:55:35Z roi $
 */

/*
 *  CycloneV FPGA UARTデバイスドライバの外部宣言
 */

#ifndef _FPGAUART_H_
#define _FPGAUART_H_

#include "pinmode.h"
#include "socfpga_cv.h"

#ifdef __cplusplus
 extern "C" {
#endif

/*
 *  FPGAUARTポート定義
 */
#define FPGAUART1_PORTID        1
#define FPGAUART2_PORTID        2
#define NUM_FPGAUARTPORT        1


#define INHNO_FPGAUART1  (0x00010000 | INTNO_F2S_FPGA_IRQ0)	/* 割込みハンドラ番号 */
#define INTNO_FPGAUART1  INTNO_F2S_FPGA_IRQ0	/* 割込み番号 */
#define INTPRI_FPGAUART1 -5			/* 割込み優先度 */
#define INTATR_FPGAUART1 0			/* 割込み属性 */


#ifndef TOPPERS_MACRO_ONLY

/*
 *  I2Cハードウェア設定構造体
 */
typedef struct _FPGAUART_PortControlBlock{
	uint32_t              base;
	uint32_t              intno;
	uint32_t              rtspin;
	uint32_t              ctspin;
} FPGAUART_PortControlBlock;

/*
 *  QUEUEバッファ定義
 */
typedef struct _queue_s {
	uint16_t    size;
	uint16_t    top;
	uint16_t    tail;
	uint16_t    limit;
	uint8_t     *buffer;
} queue_t;

/*
 *  拡張UARTハンドラ定義
 */
typedef struct fpgauart_struct FPGAUART_Handle_t;
struct fpgauart_struct {
	uint32_t    base;
	const FPGAUART_PortControlBlock *pcb;
	uint32_t	ctrl;
	queue_t     *rque;
	queue_t     *tque;
	void        (*rxcallback)(FPGAUART_Handle_t * huart);	/* 受信終了コールバック関数 */
};

#endif /* TOPPERS_MACRO_ONLY */

/*
 *  拡張UART関数のプロトタイプ宣言
 */
#ifndef TOPPERS_MACRO_ONLY

extern FPGAUART_Handle_t *fpgauart_init(ID portid);
extern ER   fpgauart_deinit(FPGAUART_Handle_t *uart);
extern int  fpgauart_recev(FPGAUART_Handle_t *uart, unsigned char *buf, int len);
extern int  fpgauart_send(FPGAUART_Handle_t *uart, unsigned char *buf, int len);
extern int  fpgauart_recsize(FPGAUART_Handle_t *uart);
extern bool_t fpgauart_cts_sens(FPGAUART_Handle_t *huart);
extern void fpgauart_isr(intptr_t exinf);


#endif /* TOPPERS_MACRO_ONLY */

#ifdef __cplusplus
}
#endif

#endif	/* _FPGAUART_H_ */

