/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 * 
 *  Copyright (C) 2008-2011 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2016-2017 by TOPPERS PROJECT Educational Working Group.
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: i2c.h 698 2016-12-19 18:25:10Z roi $
 */
/*
 * 
 *  CycloneV HPS I2Cデバイスドライバの外部宣言
 *
 */

#ifndef _I2C_H_
#define _I2C_H_

#ifdef __cplusplus
 extern "C" {
#endif

#include "socfpga_cv.h"

/*
 *  I2Cポート定義
 */
#define I2C1_PORTID             1
#define I2C2_PORTID             2
#define I2C3_PORTID             3
#define I2C4_PORTID             4
#define NUM_I2CPORT             4

#define ALT_I2C_RX_FIFO_NUM_ENTRIES     64
#define ALT_I2C_TX_FIFO_NUM_ENTRIES     64

/*
 *  I2Cリスタートイネーブル
 */
#define I2C_RESTART_DISABLE      0x00000000
#define I2C_RESATRT_ENABLE       I2C_CON_IC_RESTART_EN

/*
 *  I2Cアドレッシングモード
 */
#define I2C_ADDRESSINGMODE_7BIT  0x00000000
#define I2C_ADDRESSINGMODE_10BIT 0x00000001

/*
 *  I2CスレーブNACアフターデータジェネレートモード
 */
#define I2C_NACK_E_NORMAL        0x00000000
#define I2C_NACK_E_AFTERDBYTE    I2C_SLV_DATA_NACK_ONLY

#define I2C_SS_DEFAULT_SPKLEN   11			/* SSデフォルトSPKLEN */
#define I2C_FS_DEFAULT_SPKLEN   4			/* FSデフォルトSPKLEN */

/*
 *  I2C状態定義
 */
#define I2C_STATUS_RESET    0x00			/* I2C リセット状態 */
#define I2C_STATUS_READY    0x01			/* I2C レディ状態 */
#define I2C_STATUS_BUSY     0x02			/* I2C ビジィ状態 */
#define I2C_STATUS_BUSY_TF  0x52			/* I2C 送信中(アドレス) */
#define I2C_STATUS_BUSY_TX  0x62			/* I2C 送信中 */
#define I2C_STATUS_BUSY_RF  0x72			/* I2C 受信中(アドレス) */
#define I2C_STATUS_BUSY_RX  0x82			/* I2C 受信中 */

/*
 *  I2Cエラー定義
 */
#define I2C_ERROR_NONE      0x00000000		/* No error              */
#define I2C_ERROR_RXUNDR    0x00000001		/* RX under error        */
#define I2C_ERROR_RXOVR     0x00000002		/* RX over error         */
#define I2C_ERROR_TXABOUT   0x00000004		/* TX about error        */
#define I2C_ERROR_TXOVR     0x00000008		/* TX over error         */
#define I2C_ERROR_TIMEOUT   0x00000020		/* Timeout error         */
#define I2C_ERROR_SIZE      0x00000040		/* Size Management error */

/*
 *  I2Cハードウェア設定構造体
 */

typedef struct _I2C_PortControlBlock{
	uint32_t              base;
	uint32_t              intno;
	uint32_t              resetbit;
} I2C_PortControlBlock;

/*
 *  I2Cコンフュギュレーション構造体定義
 */
typedef struct
{
	uint32_t              ClockSpeed;		/* I2C クロックスピード */
	uint32_t              AddressingMode;	/* I2C アドレッシングモード */
	uint32_t              OwnAddress1;		/* I2C 最初のデバイスの自分のアドレス */
	uint32_t              GeneralCallMode;	/* I2C ジェネラル・コール・モード */
	uint16_t              SdaHoldClock;		/* I2C SDAのホールドクロック数 */
    uint8_t               restart_enable;	/* I2C マスターリスタート有効設定 */
    uint8_t               fs_spklen;		/* I2C SS/FSモード最長SPIKEクロック設定 */
    uint8_t               nack_enable;		/* I2CスレーブモードNACK生成有効設定 */
	int                   semid;			/* I2C 通信用セマフォ値 */
	int                   semlock;			/* I2C ロックセマフォ値 */
}I2C_Init_t;


/*
 *  I2Cハンドラ定義
 */
typedef struct I2C_Handle I2C_Handle_t;
struct I2C_Handle
{
	uint32_t              base;			/* HPS I2C ベースアドレス */
	I2C_Init_t            Init;			/* I2C 初期化データ */
    uint32_t              clock_freq;	/* I2C 入力クロック値 */
	uint8_t               *pBuffPtr;	/* I2C 通信バッファへのポインタ */
	uint16_t              XferSize;		/* I2C 通信サイズ */
	volatile uint16_t     XferCount;	/* I2C 通信カウンタ */
	volatile uint16_t     XferFifo;		/* I2C 通信FIFOカウンタ */
    uint16_t              last_target;	/* I2C master 現在のスレーブアドレス */
	void                  (*writecallback)(I2C_Handle_t * hi2c);	/* 送信終了コールバック関数 */
	void                  (*readcallback)(I2C_Handle_t * hi2c);		/* 受信終了コールバック関数 */
	void                  (*errorcallback)(I2C_Handle_t * hi2c);	/* エラーコールバック関数 */
	volatile uint32_t     Status;		/* I2C 実行状態 */
	volatile uint32_t     ErrorCode;	/* I2C Error code */
	int16_t               PortID;		/* I2C PortID */
	uint8_t               issue_restart;/* I2C リスタートモード */
	uint8_t               issue_stop;	/* I2C ストップモード */
};



extern I2C_Handle_t *i2c_init(ID port, I2C_Init_t *ii2c);
extern ER i2c_deinit(I2C_Handle_t *hi2c);
extern ER i2c_reset(I2C_Handle_t * hi2c);
extern bool_t i2c_is_busy(I2C_Handle_t *i2c_dev);
extern ER i2c_slavewrite(I2C_Handle_t *hi2c, uint8_t *data, const uint16_t Size);
extern ER i2c_slaveread(I2C_Handle_t *hi2c, uint8_t *pData, const uint16_t Size);
extern ER i2c_memwrite(I2C_Handle_t *hi2c, uint16_t devaddr, uint16_t addr, uint16_t addrsize, const void * data, uint32_t Size);
extern ER i2c_memread(I2C_Handle_t *hi2c, uint16_t devaddr, uint16_t addr, uint16_t addrsize, void * data, uint32_t Size);
extern ER i2c_int_enable(I2C_Handle_t *hi2c, const uint32_t mask);
extern ER i2c_int_disable(I2C_Handle_t *hi2c, const uint32_t mask);
extern void i2c_hps_isr(intptr_t exinf);


#ifdef __cplusplus
}
#endif

#endif	/* _I2C_H_ */

