/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2015-2016 by TOPPERS PROJECT Educational Working Group.
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  $Id: usb_otg.c 2416 2016-12-10 16:12:20Z roi $
 */

/* 
 *  USB_OTGドライバの本体
 */

#include "kernel_impl.h"
#include <t_syslog.h>
#include <t_stdlib.h>
#include <string.h>
#include <sil.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include <target_syssvc.h>
#include "device.h"
#include "usb_otg.h"

#define sil_orw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) | (b))
#define sil_andw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) & ~(b))
#define sil_andxw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) & (b))
#define sil_modw_mem(a, b, c)	sil_wrw_mem((a), (sil_rew_mem(a) & (~b)) | (c))

#define get_usbmode(a)          (sil_rew_mem((uint32_t *)(USBOTG_BASE((a))+TOFF_USBO_GINTSTS)) & 1)
#define get_int_status(a, b)    (sil_rew_mem((uint32_t *)(USBOTG_BASE((a))+TOFF_USBO_GINTSTS)) & (b))
#define get_haint_status(a)     (sil_rew_mem((uint32_t *)(USBH_BASE((a))+TOFF_USBH_HAINT)) & 0xFFFF)
#define clear_host_int(a, b)    sil_wrw_mem((uint32_t *)(USBOTG_BASE((a))+TOFF_USBO_GINTSTS), (b))
#define clear_dev_int(a, b)		sil_andxw_mem((uint32_t *)(USBOTG_BASE((a))+TOFF_USBO_GINTSTS), (b))
#define clear_hc_int(a, b, c)   sil_wrw_mem((uint32_t *)(USBHC_BASE((a), (b))+TOFF_USBHC_HCINT), (c))
#define clear_outep_int(a, b, c) sil_wrw_mem((uint32_t *)(USBOEP_BASE((a), (b))+TOFF_USBEO_DOEPINT), (c))
#define clear_inep_int(a, b, c) sil_wrw_mem((uint32_t *)(USBIEP_BASE((a), (b))+TOFF_USBEI_DIEPINT), (c))
#define get_inep_int_status(a)  ((sil_rew_mem((uint32_t *)(USBD_BASE((a))+TOFF_USBD_DAINT)) \
								& sil_rew_mem((uint32_t *)(USBD_BASE((a))+TOFF_USBD_DAINTMSK))) & 0xFFFF)
#define get_outep_int_status(a) (((sil_rew_mem((uint32_t *)(USBD_BASE((a))+TOFF_USBD_DAINT)) \
								& sil_rew_mem((uint32_t *)(USBD_BASE((a))+TOFF_USBD_DAINTMSK))) & 0xFFFF0000) >> 16)
#define enter_spinlock(a)       ((a) != 0) ? loc_spn((a)) : E_OK
#define release_spinlock(a)     ((a) != 0) ? unl_spn((a)) : E_OK
#define enter_ispinlock(a)      ((a) != 0) ? iloc_spn((a)) : E_OK
#define release_ispinlock(a)    ((a) != 0) ? iunl_spn((a)) : E_OK

#define get_packet_data(s)      ((s[0] & 0xff) | ((s[1] & 0xff)<<8) | ((s[2] & 0xff)<<16) | ((s[3] & 0xff)<<24))

/*
 *  USB用 AF10定義
 */
#define GPIO_AF10_OTG_FS        ((uint8_t)0x0AU)  /* OTG_FS Alternate Function mapping */
#define GPIO_AF10_OTG_HS        ((uint8_t)0x0AU)  /* OTG_HS Alternate Function mapping */

/*
 *  デフォルトFIFO設定
 */
#define DEFAULT_HS_GRXFSIZ      2048
#define DEFAULT_HS_TXFIFO0_SIZ  1024

/*
 *  パケットサイズ定義
 */
#define USB_OTG_HS_MAX_PACKET_SIZE           512
#define USB_OTG_FS_MAX_PACKET_SIZE           64
#define USB_OTG_MAX_EP0_SIZE                 64

/*
 *  PHY周波数定義
 */
#define DSTS_ENUMSPD_HS_PHY_30MHZ_OR_60MHZ     (0 << 1)
#define DSTS_ENUMSPD_FS_PHY_30MHZ_OR_60MHZ     (1 << 1)
#define DSTS_ENUMSPD_LS_PHY_6MHZ               (2 << 1)
#define DSTS_ENUMSPD_FS_PHY_48MHZ              (3 << 1)

/*
 * フレームインターバル定義
 */
#define DCFG_FRAME_INTERVAL_80                 0
#define DCFG_FRAME_INTERVAL_85                 1
#define DCFG_FRAME_INTERVAL_90                 2
#define DCFG_FRAME_INTERVAL_95                 3

/*
 *  パケットステータス定義
 */
#define STS_GOUT_NAK                           1
#define STS_DATA_UPDT                          2
#define STS_XFER_COMP                          3
#define STS_SETUP_COMP                         4
#define STS_SETUP_UPDT                         6

/*
 *  PHYスピード定義
 */
#define HPRT0_PRTSPD_HIGH_SPEED                0
#define HPRT0_PRTSPD_FULL_SPEED                1
#define HPRT0_PRTSPD_LOW_SPEED                 2

/*
 *  ホストクラスエンドポイントタイプ定義
 */
#define HCCHAR_CTRL                            0
#define HCCHAR_ISOC                            1
#define HCCHAR_BULK                            2
#define HCCHAR_INTR                            3


#define GRXSTS_PKTSTS_IN                       2
#define GRXSTS_PKTSTS_IN_XFER_COMP             3
#define GRXSTS_PKTSTS_DATA_TOGGLE_ERR          5
#define GRXSTS_PKTSTS_CH_HALTED                7

#define RESET_TIMEOUT           (1000*100)		/* 1ms */
#define HALT_TIMEOUT            (10*100)		/* 0.01ms */
#define USB_OTG_FS_WAKEUP_EXTI_LINE         0x00040000  /*!< External interrupt line 18 Connected to the USB FS EXTI Line */

/*
 *  HBSTLENGTH
 */
#define GAHBCFG_HBSTLEN_SINGLE                 0
#define GAHBCFG_HBSTLEN_INCR                   USB_OTG_GAHBCFG_HBSTLEN_0
#define GAHBCFG_HBSTLEN_INCR4                  (USB_OTG_GAHBCFG_HBSTLEN_0 | USB_OTG_GAHBCFG_HBSTLEN_1) //3
#define GAHBCFG_HBSTLEN_INCR8                  (USB_OTG_GAHBCFG_HBSTLEN_0 | USB_OTG_GAHBCFG_HBSTLEN_2) //  5
#define GAHBCFG_HBSTLEN_INCR16                 (USB_OTG_GAHBCFG_HBSTLEN_0 | USB_OTG_GAHBCFG_HBSTLEN_1 | USB_OTG_GAHBCFG_HBSTLEN_2) //7

/*
 *  PHYIF
 */
#define GUSBCFG_PHYIF16                        USB_OTG_GUSBCFG_PHYIF
#define GUSBCFG_PHYIF8                         0

/*
 *  VBUS定義
 */
//#define DWC2_PHY_ULPI_INTERNAL_VBUS            0
//#define DWC2_PHY_ULPI_EXTERNAL_VBUS            1

/*
 *  OP MODE値
 */
#define GHWCFG2_OP_MODE_HNP_SRP_CAPABLE        0
#define GHWCFG2_OP_MODE_SRP_ONLY_CAPABLE       1
#define GHWCFG2_OP_MODE_NO_HNP_SRP_CAPABLE     2
#define GHWCFG2_OP_MODE_SRP_CAPABLE_DEVICE     3
#define GHWCFG2_OP_MODE_NO_SRP_CAPABLE_DEVICE  4
#define GHWCFG2_OP_MODE_SRP_CAPABLE_HOST       5
#define GHWCFG2_OP_MODE_NO_SRP_CAPABLE_HOST    6
#define GHWCFG2_OP_MODE_UNDEFINED              7

/*
 *  ARCH値
 */
#define GHWCFG2_SLAVE_ONLY_ARCH                0
#define GHWCFG2_EXT_DMA_ARCH                   1
#define GHWCFG2_INT_DMA_ARCH                   2

/*
 *  PHYタイプ
 */
#define GHWCFG2_HS_PHY_TYPE_NOT_SUPPORTED      0
#define GHWCFG2_HS_PHY_TYPE_UTMI               1
#define GHWCFG2_HS_PHY_TYPE_ULPI               2
#define GHWCFG2_HS_PHY_TYPE_UTMI_ULPI          3

/*
 *  PHYデータ長
 */
#define GHWCFG4_UTMI_PHY_DATA_WIDTH_8          0
#define GHWCFG4_UTMI_PHY_DATA_WIDTH_16         1
#define GHWCFG4_UTMI_PHY_DATA_WIDTH_8_OR_16    2

#define GHWCFG2_OP_MODE                        0x00000007
#define GHWCFG2_ARCHITECTURE                   0x00000018
#define GHWCFG2_HS_PHY_TYPE                    0x000000c0
#define GHWCFG2_FS_PHY_TYPE                    0x00000300
#define GHWCFG2_NUM_DEV_EP                     0x00003c00
#define GHWCFG2_NUM_HOST_CHAN                  0x0003c000
#define GHWCFG2_DYNAMIC_FIFO                   0x00080000
#define GHWCFG2_NONPERIO_TX_Q_DEPTH            0x00c00000
#define GHWCFG2_HOST_PERIO_TX_Q_DEPTH          0x03000000
#define GHWCFG2_DEV_TOKEN_Q_DEPTH              0x7c000000

#define GHWCFG3_XFER_SIZE_CNTR_WIDTH           0x0000000f
#define GHWCFG3_PACKET_SIZE_CNTR_WIDTH         0x00000070
#define GHWCFG3_I2C                            0x00000100
#define GHWCFG3_DFIFO_DEPTH                    0xffff0000

#define GHWCFG4_NUM_DEV_PERIO_IN_EP            0x0000000f
#define GHWCFG4_POWER_OPTIMIZ                  0x00000010
#define GHWCFG4_UTMI_PHY_DATA_WIDTH            0x0000c000
#define GHWCFG4_DED_FIFO_EN                    0x02000000
#define GHWCFG4_DESC_DMA                       0x40000000

#define GAHBCFG_CTRL_MASK (USB_OTG_GAHBCFG_PTXFELVL | USB_OTG_GAHBCFG_TXFELVL | \
						   USB_OTG_GAHBCFG_DMAEN | USB_OTG_GAHBCFG_GINT)
#define HCINTMSK_RESERVED14_31                 (0x3ffff << 14)

/*
 *  USBポートIDから管理ブロックを取り出すためのマクロ
 */
#define INDEX_USB(usbid)        ((uint_t)((usbid) - 1))


#ifndef EXT_CLK_CLKSC1_FREQCUR
#define EXT_CLK_CLKSC1_FREQCUR                 25000000
#endif
#ifndef EXT_CLK_CLKSC2_FREQCUR
#define EXT_CLK_CLKSC2_FREQCUR	               25000000
#endif
#ifndef EXT_CLK_PERIPH_FREQCUR
#define EXT_CLK_PERIPH_FREQCUR                 0
#endif

/*
 *  CLOCK SOURCE
 */
#define CLKMGR_PERPLL_VCO_PSRC_E_EOSC1         0x0	/* eosc1_clk */
#define CLKMGR_PERPLL_VCO_PSRC_E_EOSC2         0x1	/* eosc2_clk */
#define CLKMGR_PERPLL_VCO_PSRC_E_F2S_PERIPH_REF 0x2	/* f2s_periph_ref_clk */

#define CLKMGR_PERPLL_DIV_USBCLK_E_DIV16       0x4

static const uint32_t default_pcd_tx_fifo_size[] = {256, 256, 256, 256, 768, 768, 768, 768};

/*
 *  USBポート設定テーブル
 */
static const USB_PortControlBlock usb_pcb[NUM_USBPORT] = {
  {	TADR_USB_OTG0_BASE, INTNO_USB0_IRQ, RSTMGR_PERMODRST_USB0 },
  {	TADR_USB_OTG1_BASE, INTNO_USB1_IRQ, RSTMGR_PERMODRST_USB1 }
};

static USB_OTG_Handle_t Husb[NUM_USBPORT];

static void usbo_pcd_control(USB_OTG_Handle_t *husb, int is_active);

static uint32_t
get_usb_clock(void)
{
	uint32_t temp = 0;
	uint64_t temp1 = 0;
    uint64_t numer = 0;
    uint64_t denom = 1;

	if((sil_rew_mem((uint32_t *)(TADR_CLKMGR_PERPLL_BASE+TOFF_CLKMGR_PERPLL_EN)) & CLKMGR_PERPLL_EN_USBCLK) != 0){

		temp = sil_rew_mem((uint32_t *)(TADR_CLKMGR_MGR_BASE+TOFF_CLKMGR_BYPASS));
		if((temp & CLKMGR_BYPASS_PERPLL) != 0){
			if((temp & CLKMGR_BYPASS_PERPLLSRC) == 0)
				temp = EXT_CLK_CLKSC1_FREQCUR;
			else
				temp = EXT_CLK_PERIPH_FREQCUR;
		}
		else{
			temp = sil_rew_mem((uint32_t *)(TADR_CLKMGR_PERPLL_BASE+TOFF_CLKMGR_PERPLL_VCO));
			numer = (temp & CLKMGR_PERPLL_VCO_NUMER) >> 3;
			denom = (temp & CLKMGR_PERPLL_VCO_DENOM) >> 16;
			temp = (temp & CLKMGR_PERPLL_VCO_PSRC) >> 22;
			if (temp == CLKMGR_PERPLL_VCO_PSRC_E_EOSC1)
				temp1 = (uint64_t) EXT_CLK_CLKSC1_FREQCUR;
			else if (temp == CLKMGR_PERPLL_VCO_PSRC_E_EOSC2)
				temp1 = (uint64_t) EXT_CLK_CLKSC2_FREQCUR;
			else if (temp == CLKMGR_PERPLL_VCO_PSRC_E_F2S_PERIPH_REF)
				temp1 = (uint64_t) EXT_CLK_PERIPH_FREQCUR;

			if(temp1 != 0){
				temp1 *= (numer + 1);
				temp1 /= (denom + 1);
				if (temp1 <= UINT32_MAX)
					temp = (uint32_t)temp1;
				else
					return 0;
			}
		}

		numer = (uint64_t) temp;
		temp = (sil_rew_mem((uint32_t *)(TADR_CLKMGR_PERPLL_BASE+TOFF_CLKMGR_PERPLL_PERBASECLK)) & CLKMGR_PERPLL_PERBASECLK_CNT) + 1;
		denom = (uint64_t) temp;

		temp = sil_rew_mem((uint32_t *)(TADR_CLKMGR_PERPLL_BASE+TOFF_CLKMGR_PERPLL_DIV)) & CLKMGR_PERPLL_DIV_USBCLK;
		if (temp <= CLKMGR_PERPLL_DIV_USBCLK_E_DIV16)
			temp = 1 << temp;
		else
			return 0;
		denom = denom * (uint64_t)temp;

		if(denom > 0){
			numer /= denom;
			if(numer <= UINT32_MAX)
				return (uint32_t)numer;
		}
    	return 0;
	}
	else
		return 0;
}

/*
 *  必要なら強制モードを設定する
 *  強制モードならtrueを返す
 */
static bool_t
usbo_force_mode_if_needed(USB_OTG_Handle_t *husb, bool_t host)
{
	uint32_t otgmode = get_usbmode(husb->base);
	uint32_t reqmode = husb->Init.usb_otg_mode;
	uint32_t gusbcfg;
	uint32_t set, clear;

	if (host && (otgmode == USB_OTG_MODE_HOST || reqmode == USB_OTG_MODE_DEVICE))
		return false;
	else if (!host && (otgmode == USB_OTG_MODE_DEVICE || reqmode == USB_OTG_MODE_HOST))
		return false;

	gusbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GUSBCFG));
	set = host ? USB_OTG_GUSBCFG_FHMOD : USB_OTG_GUSBCFG_FDMOD;
	clear = host ? USB_OTG_GUSBCFG_FDMOD : USB_OTG_GUSBCFG_FHMOD;
	gusbcfg &= ~clear;
	gusbcfg |= set;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GUSBCFG), gusbcfg);

	dly_tsk(25);
	return true;
}

/*
 *  強制モードをクリアする
 */
static void
usbo_clear_force_mode(USB_OTG_Handle_t *husb)
{
	uint32_t gusbcfg;

	gusbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GUSBCFG));
	gusbcfg &= ~USB_OTG_GUSBCFG_FHMOD;
	gusbcfg &= ~USB_OTG_GUSBCFG_FDMOD;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GUSBCFG), gusbcfg);

	/*
	 *  設定変更による待ち
	 */
	dly_tsk(25);
}

/*
 *  USB-OTGモードチェック
 */
static ER
usbo_check_dr_mode(USB_OTG_Handle_t *husb)
{
	uint32_t mode = husb->Init.usb_otg_mode;;
	uint32_t op_mode;

	mode = husb->Init.usb_otg_mode;
	op_mode = sil_rew_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GHWCFG2)) & USB_OTG_GHWCFG2_OP_MODE;
	if(op_mode == USB_OTG_GHWCFG2_SRP_CAPABLE_DEVICE || op_mode == USB_OTG_GHWCFG2_NO_SRP_CAPABLE_DEVICE){
		/* device */
		if(mode == USB_OTG_MODE_HOST){
			syslog_0(LOG_ERROR, "Controller does not support host mode.");
			return E_NOSPT;
		}
		mode = USB_OTG_MODE_DEVICE;
	}
	else if(op_mode == USB_OTG_GHWCFG2_SRP_CAPABLE_HOST || op_mode == USB_OTG_GHWCFG2_NO_SRP_CAPABLE_HOST){
		if(mode == USB_OTG_MODE_DEVICE){
			syslog_0(LOG_ERROR, "Controller does not support device mode.");
			return E_NOSPT;
		}
		mode = USB_OTG_MODE_HOST;
	}
	if(mode != husb->Init.usb_otg_mode){
		syslog_1(LOG_ERROR, "Configuration mismatch. dr_mode forced to %s", (mode == USB_OTG_MODE_HOST ? "host" : "device"));
		return E_OBJ;
	}
	return E_OK;
}

/*
 *  USB-OTGハードウェア情報取得
 */
static ER
usbo_get_hwparam(USB_OTG_Handle_t *husb)
{
	USB_OTG_HwParam_t *hw = &husb->hw_params;
	unsigned width;
	uint32_t base = husb->base;
	uint32_t hwcfg1, hwcfg2, hwcfg3, hwcfg4;
	uint32_t grxfsiz, gnptxfsiz, hptxfsiz;
	bool_t forced;

	/*
	 * Attempt to ensure this device is really a DWC_otg Controller.
	 * Read and verify the GSNPSID register contents. The value should be
	 * 0x45f42xxx or 0x45f43xxx, which corresponds to either "OT2" or "OT3",
	 * as in "OTG version 2.xx" or "OTG version 3.xx".
	 */
	hw->snpsid = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GSNPSID));
	if ((hw->snpsid & 0xfffff000) != 0x4f542000 &&
	    (hw->snpsid & 0xfffff000) != 0x4f543000) {
		syslog_1(LOG_ERROR, "Bad value for GSNPSID: 0x%08x", hw->snpsid);
		return E_OBJ;
	}

	syslog_5(LOG_NOTICE, "Core Release: %1x.%1x%1x%1x (snpsid=%x)",
		hw->snpsid >> 12 & 0xf, hw->snpsid >> 8 & 0xf,
		hw->snpsid >> 4 & 0xf, hw->snpsid & 0xf, hw->snpsid);

	hwcfg1 = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GHWCFG1));
	hwcfg2 = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GHWCFG2));
	hwcfg3 = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GHWCFG3));
	hwcfg4 = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GHWCFG4));
	grxfsiz = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRXFSIZ));
	gnptxfsiz = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_DIEPTXF0_HNPTXFSIZ));
	hptxfsiz = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_HPTXFSIZ));

	/*
	 *  ホスト情報取得
	 */
	if(husb->Init.usb_otg_mode != USB_OTG_MODE_DEVICE){
		forced = usbo_force_mode_if_needed(husb, true);
		hw->host_nperio_tx_fifo_size = (gnptxfsiz & USB_OTG_NPTXFD) >> 16;
		hw->host_perio_tx_fifo_size = (hptxfsiz & USB_OTG_HPTXFSIZ_PTXFD) >> 16;
		if(forced)
			usbo_clear_force_mode(husb);
	}

	/*
	 *  デバイス情報取得
	 */
	if(husb->Init.usb_otg_mode != USB_OTG_MODE_HOST){
		forced = usbo_force_mode_if_needed(husb, false);
		hw->dev_nperio_tx_fifo_size = (gnptxfsiz & USB_OTG_NPTXFD) >> 16;
		if(forced)
			usbo_clear_force_mode(husb);
	}

	/* hwcfg1 */
	hw->dev_ep_dirs = hwcfg1;

	/* hwcfg2 */
	hw->op_mode = hwcfg2 & GHWCFG2_OP_MODE;
	hw->arch = (hwcfg2 & GHWCFG2_ARCHITECTURE) >> 3;
	hw->enable_dynamic_fifo = !!(hwcfg2 & GHWCFG2_DYNAMIC_FIFO);
	hw->host_channels = 1 + ((hwcfg2 & GHWCFG2_NUM_HOST_CHAN) >> 14);
	hw->hs_phy_type = (hwcfg2 & GHWCFG2_HS_PHY_TYPE) >> 6;
	hw->fs_phy_type = (hwcfg2 & GHWCFG2_FS_PHY_TYPE) >> 8;
	hw->num_dev_ep = (hwcfg2 & GHWCFG2_NUM_DEV_EP) >> 10;
	hw->nperio_tx_q_depth = ((hwcfg2 & GHWCFG2_NONPERIO_TX_Q_DEPTH) >> 22) << 1;
	hw->host_perio_tx_q_depth = ((hwcfg2 & GHWCFG2_HOST_PERIO_TX_Q_DEPTH) >> 24) << 1;
	hw->dev_token_q_depth = (hwcfg2 & GHWCFG2_DEV_TOKEN_Q_DEPTH) >> 26;

	/* hwcfg3 */
	width = (hwcfg3 & GHWCFG3_XFER_SIZE_CNTR_WIDTH);
	hw->max_transfer_size = (1 << (width + 11)) - 1;
	width = (hwcfg3 & GHWCFG3_PACKET_SIZE_CNTR_WIDTH) >> 4;
	hw->max_packet_count = (1 << (width + 4)) - 1;
	hw->i2c_enable = !!(hwcfg3 & GHWCFG3_I2C);
	hw->total_fifo_size = (hwcfg3 & GHWCFG3_DFIFO_DEPTH) >> 16;

	/* hwcfg4 */
	hw->en_multiple_tx_fifo = !!(hwcfg4 & GHWCFG4_DED_FIFO_EN);
	hw->num_dev_perio_in_ep = (hwcfg4 & GHWCFG4_NUM_DEV_PERIO_IN_EP);
	hw->dma_desc_enable = !!(hwcfg4 & GHWCFG4_DESC_DMA);
	hw->power_optimized = !!(hwcfg4 & GHWCFG4_POWER_OPTIMIZ);
	hw->utmi_phy_data_width = (hwcfg4 & GHWCFG4_UTMI_PHY_DATA_WIDTH) >> 14;

	/* fifo sizes */
	hw->host_rx_fifo_size = grxfsiz & USB_OTG_GRXFSIZ_RXFD;

#if 1	/* ROI DEBUG */
	syslog_0(LOG_NOTICE, "Detected values from hardware:");
	syslog_1(LOG_NOTICE, "  op_mode=%d", hw->op_mode);
	syslog_1(LOG_NOTICE, "  arch=%d", hw->arch);
	syslog_1(LOG_NOTICE, "  dma_desc_enable=%d", hw->dma_desc_enable);
	syslog_1(LOG_NOTICE, "  power_optimized=%d", hw->power_optimized);
	syslog_1(LOG_NOTICE, "  i2c_enable=%d", hw->i2c_enable);
	syslog_1(LOG_NOTICE, "  hs_phy_type=%d", hw->hs_phy_type);
	syslog_1(LOG_NOTICE, "  fs_phy_type=%d", hw->fs_phy_type);
	dly_tsk(200);
	syslog_1(LOG_NOTICE, "  utmi_phy_data_width=%d", hw->utmi_phy_data_width);
	syslog_1(LOG_NOTICE, "  num_dev_ep=%d", hw->num_dev_ep);
	syslog_1(LOG_NOTICE, "  num_dev_perio_in_ep=%d", hw->num_dev_perio_in_ep);
	syslog_1(LOG_NOTICE, "  host_channels=%d", hw->host_channels);
	syslog_1(LOG_NOTICE, "  max_transfer_size=%d", hw->max_transfer_size);
	syslog_1(LOG_NOTICE, "  max_packet_count=%d", hw->max_packet_count);
	syslog_1(LOG_NOTICE, "  nperio_tx_q_depth=0x%0x", hw->nperio_tx_q_depth);
	syslog_1(LOG_NOTICE, "  host_perio_tx_q_depth=0x%0x", hw->host_perio_tx_q_depth);
	syslog_1(LOG_NOTICE, "  dev_token_q_depth=0x%0x", hw->dev_token_q_depth);
	syslog_1(LOG_NOTICE, "  enable_dynamic_fifo=%d", hw->enable_dynamic_fifo);
	syslog_1(LOG_NOTICE, "  en_multiple_tx_fifo=%d", hw->en_multiple_tx_fifo);
	syslog_1(LOG_NOTICE, "  total_fifo_size=%d", hw->total_fifo_size);
	syslog_1(LOG_NOTICE, "  host_rx_fifo_size=%d", hw->host_rx_fifo_size);
	syslog_1(LOG_NOTICE, "  host_nperio_tx_fifo_size=%d", hw->host_nperio_tx_fifo_size);
	syslog_1(LOG_NOTICE, "  host_perio_tx_fifo_size=%d", hw->host_perio_tx_fifo_size);
	dly_tsk(200);
#endif	/* ROI DEBUG */
	return E_OK;
}

/*
 *  USB-OTG実行パラメータ設定
 */
static ER
usbo_set_parameters(USB_OTG_Handle_t *husb)
{
	USB_OTG_HwParam_t *hw = &husb->hw_params;
	int val = 0;
	ER ercd = E_OK;

	/* param_otg_cap */
	switch (hw->op_mode) {
	case GHWCFG2_OP_MODE_HNP_SRP_CAPABLE:
		val = DWC2_CAP_PARAM_HNP_SRP_CAPABLE;
		break;
	case GHWCFG2_OP_MODE_SRP_ONLY_CAPABLE:
	case GHWCFG2_OP_MODE_SRP_CAPABLE_DEVICE:
	case GHWCFG2_OP_MODE_SRP_CAPABLE_HOST:
		val = DWC2_CAP_PARAM_SRP_ONLY_CAPABLE;
		break;
	default:
		val = DWC2_CAP_PARAM_NO_HNP_SRP_CAPABLE;
		break;
	}
	husb->otg_cap = val;

	/* enable_dynamic_fifo */
	husb->enable_dynamic_fifo = hw->enable_dynamic_fifo;
	/* en_multiple_tx_fifo(notuse) */
	husb->en_multiple_tx_fifo = 0;
	/* host_rx_fifo_size */
	husb->host_rx_fifo_size = hw->host_rx_fifo_size;
	/* host_nperio_tx_fifo_size */
	husb->host_nperio_tx_fifo_size = hw->host_nperio_tx_fifo_size;
	/* host_perio_tx_fifo_size */
	husb->host_perio_tx_fifo_size = hw->host_perio_tx_fifo_size;
	/* max_transfer_size */
	husb->max_transfer_size = hw->max_transfer_size;
	/* max_packet_count */
	husb->max_packet_count = hw->max_packet_count;

	/* phy_type */
	val = PHY_TYPE_PARAM_FS;
	if(hw->hs_phy_type != GHWCFG2_HS_PHY_TYPE_NOT_SUPPORTED) {
		if (hw->hs_phy_type == GHWCFG2_HS_PHY_TYPE_UTMI ||
		    hw->hs_phy_type == GHWCFG2_HS_PHY_TYPE_UTMI_ULPI)
			val = PHY_TYPE_PARAM_UTMI;
		else
			val = PHY_TYPE_PARAM_ULPI;
	}
	husb->phy_type = val;

	/* phy_utmi_width */
	val = (hw->utmi_phy_data_width == GHWCFG4_UTMI_PHY_DATA_WIDTH_8) ? 8 : 16;
	husb->phy_utmi_width = val;
	/* hibernation */
	husb->hibernation = 0;

	if(husb->enable_dynamic_fifo != 0){

		uint32_t total_fifo_size = hw->total_fifo_size;
		uint32_t rxfsiz = husb->host_rx_fifo_size;
		uint32_t nptxfsiz = husb->host_nperio_tx_fifo_size;
		uint32_t ptxfsiz = husb->host_perio_tx_fifo_size;

		if(total_fifo_size < (rxfsiz + nptxfsiz + ptxfsiz)){
			/*
			 * For Buffer DMA mode/Scatter Gather DMA mode
			 * 2 * ((Largest Packet size / 4) + 1 + 1) + n
			 * with n = number of host channel.
			 * 2 * ((1024/4) + 2) = 516
			 */
			rxfsiz = 516 + hw->host_channels;

			/*
			 * min non-periodic tx fifo depth
			 * 2 * (largest non-periodic USB packet used / 4)
			 * 2 * (512/4) = 256
			 */
			nptxfsiz = 256;

			/*
			 * min periodic tx fifo depth
			 * (largest packet size*MC)/4
			 * (1024 * 3)/4 = 768
			 */
			ptxfsiz = 768;

			husb->host_rx_fifo_size = rxfsiz;
			husb->host_nperio_tx_fifo_size = nptxfsiz;
			husb->host_perio_tx_fifo_size = ptxfsiz;
		}
		if(total_fifo_size < (rxfsiz + nptxfsiz + ptxfsiz))
			ercd = E_OBJ;
	}

#if 0	/* ROI DEBUG */
	syslog_1(LOG_NOTICE, "Setting otg_cap to %d", husb->otg_cap);
	syslog_1(LOG_NOTICE, "Setting enable_dynamic_fifo to %d", husb->enable_dynamic_fifo);
	syslog_1(LOG_NOTICE, "Setting host_rx_fifo_size to %d", husb->host_rx_fifo_size);
	syslog_1(LOG_NOTICE, "Setting host_nperio_tx_fifo_size to %d", husb->host_nperio_tx_fifo_size);
	syslog_1(LOG_NOTICE, "Setting host_perio_tx_fifo_size to %d", husb->host_perio_tx_fifo_size);
	syslog_1(LOG_NOTICE, "Setting max_transfer_size to %d", husb->max_transfer_size);
	syslog_1(LOG_NOTICE, "Setting max_packet_count to %d", husb->max_packet_count);
	syslog_1(LOG_NOTICE, "Setting phy_type to %d", husb->phy_type);
	syslog_1(LOG_NOTICE, "Setting phy_utmi_width to %d", husb->phy_utmi_width);
	syslog_2(LOG_NOTICE, "Setting en_multiple_tx_fifo to %d hardvalue(%d)", husb->en_multiple_tx_fifo, hw->en_multiple_tx_fifo);
	syslog_1(LOG_NOTICE, "Setting hibernation to %d", husb->hibernation);
	dly_tsk(200);
#endif	/* ROI DEBUG */
	return ercd;
}

/*
 *  USB_OTGハードウェア初期化
 *  parameter1 portid USBポートID
 *  parameter2 pini   初期化構造体へのポインタ
 *  return     NULLでなければUSBハンドラへのポインタ
 */
USB_OTG_Handle_t *
usbo_init(ID portid, USB_OTG_Init_t *pini)
{
	USB_OTG_Handle_t *husb;
	uint32_t base;
	uint32_t no, clock;
	volatile uint32_t timeout = 8192;
	ER ercd = E_OK;

	if(portid < USB1_PORTID || portid > NUM_USBPORT)
		return NULL;

	no = INDEX_USB(portid);
	husb = &Husb[no];
	memcpy(&husb->Init, pini, sizeof(USB_OTG_Init_t));

	husb->base = usb_pcb[no].base;
	husb->resetbit = usb_pcb[no].resetbit;

	/*
	 *  FIFO設定値
	 */
	husb->pcd_rx_fifo_sz      = DEFAULT_HS_GRXFSIZ;
	husb->pcd_np_g_tx_fifo_sz = DEFAULT_HS_TXFIFO0_SIZ;
	memcpy(&husb->pcd_tx_fifo_sz[1], default_pcd_tx_fifo_size, sizeof(default_pcd_tx_fifo_size));

	clock = get_usb_clock();
	if(clock == 0){
		syslog_1(LOG_ERROR, "usbo_init:no clock port(%d) !", portid);
		return NULL;
	}

	/*
	 *  デフォルトのUTMI幅
	 */
	husb->phyif = GUSBCFG_PHYIF16;

	/*
	 *  USB-OTGモジュールリセット
	 */
	sil_orw_mem((uint32_t *)(TADR_RSTMGR_BASE+TOFF_RSTMGR_PERMODRST), husb->resetbit);
    while (timeout--)
        ;
	sil_andw_mem((uint32_t *)(TADR_RSTMGR_BASE+TOFF_RSTMGR_PERMODRST), husb->resetbit);

	ercd = usbo_check_dr_mode(husb);
	if(ercd != E_OK)
		return NULL;

	base = husb->base;
	/*
	 *  USB OTGグローバル割込み禁止
	 */
	usbo_disableglobalint(husb);

	/*
	 *  USB OTGコア部初期化
	 */
	usbo_coreinit(husb);

	/*
	 *  USB OTGモード設定
	 */
	usbo_setcurrentmode(husb);

	/*
	 *  リスタートPHYクロック
	 */
	sil_wrw_mem((uint32_t *)(USBPGC_BASE(base)), 0);

	/*
	 *  GINTのオールマスク設定
	 */
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), 0);

	/*
	 *  GINTオール割込みクリア
	 */
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTSTS), 0xFFFFFFFF);

	ercd = usbo_get_hwparam(husb);
	if(ercd != E_OK){
		syslog_1(LOG_ERROR, "usbo_init:can't gets hardware param(%d) !", ercd);
		return NULL;
	}
	if(husb->hw_params.num_dev_ep != (MAX_EPS_CHANNELS-1)){
		syslog_1(LOG_ERROR, "usbo_init:number of endpoint error(%d) !", husb->hw_params.num_dev_ep);
		return NULL;
	}
	/* Validate parameter values */
	ercd = usbo_set_parameters(husb);
	if(ercd != E_OK){
		syslog_1(LOG_ERROR, "usbo_init:can't sets execite param(%d) !", ercd);
		return NULL;
	}
	if(pini->phy_itface != USB_PHY_ULPI && husb->phy_type == PHY_TYPE_PARAM_ULPI){
		syslog_2(LOG_ERROR, "usbo_init:error PHY (%d) (%d) !", pini->phy_itface, husb->phy_type);
		return NULL;
	}
	if(husb->hw_params.arch == GHWCFG2_SLAVE_ONLY_ARCH && husb->Init.dma_enable != 0){
		syslog_1(LOG_ERROR, "usbo_init:dma mode error(%d) !", husb->Init.dma_enable);
		return NULL;
	}
	return husb;
}

/*
 *  USB OTGハードウェア停止
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_deinit(USB_OTG_Handle_t *husb)
{
	sil_orw_mem((uint32_t *)(TADR_RSTMGR_BASE+TOFF_RSTMGR_PERMODRST), husb->resetbit);
	return E_OK;
}

/*
 *  パケットデータ書き込み
 *  parameter1 base  USB OTGレジスタベースアドレス
 *  parameter2 src   書き込みデータのポインタ
 *  parameter3 ch_ep_num エンドポイント、または、チャネル番号
 *  parameter4 len   データ長
 *  parameter5 dma   DMAモード(0:none-DMA、1:DMA)
 */
static void
usbo_writePacket(uint32_t base, uint8_t *src, uint8_t ch_ep_num, uint16_t len, uint8_t dma)
{
	uint32_t count32b = 0 , i = 0;
	volatile uint8_t *p = (volatile uint8_t *)src;

	if(dma == 0U){
		count32b =  (len + 3) / 4;
		for(i = 0 ; i < count32b; i++, p += 4){
			sil_wrw_mem((uint32_t *)(USBD_FIFO(base, ch_ep_num)), get_packet_data(p));
		}
	}
}

/*
 *  パケットデータ読み込み
 *  parameter1 base USB OTGレジスタベースアドレス
 *  parameter2 dest 読み込みデータのポインタ
 *  parameter3 len  データ長
 */
static void *
usbo_readPacket(uint32_t base, uint8_t *dest, uint16_t len)
{
	uint32_t i=0U;
	uint32_t count32b = (len + 3U) / 4U;
	volatile uint32_t data;

	for(i = 0 ; i < count32b; i++, dest += 4){
		data = sil_rew_mem((uint32_t *)(USBD_FIFO(base, 0)));
		dest[0] = (data & 0xff);
		dest[1] = (data >> 8) & 0xff;
		dest[2] = (data >> 16) & 0xff;
		dest[3] = (data >> 24) & 0xff;
	}
	return ((void *)dest);
}

/*
 *  USB HOSTのカレントフレーム番号を取り出す
 *  parameter1 USBハンドラへのポインタ
 *  return     ハンドラ番号
*/
uint32_t
usbo_getcurrentframe(USB_OTG_Handle_t *husb)
{
	return (sil_rew_mem((uint32_t *)(USBH_BASE(husb->base)+TOFF_USBH_HFNUM)) & USB_OTG_HFNUM_FRNUM);
}

/*
 *  USB OTGの機能設定を行う
 *  parameter1 husb  USBハンドラへのポインタ
 *     usb_otg_modeの設定：USB_OTG_MODE_DEVICE(USB DEVICE固定)
 *                         USB_OTG_MODE_HOST(USB HOST固定)
 *                         USB_OTG_MODE_DRD(デュアルモード)
 *  return     常にE_OK
 */
ER
usbo_setcurrentmode(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t gusbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GUSBCFG));

	gusbcfg &= ~(USB_OTG_GUSBCFG_FHMOD | USB_OTG_GUSBCFG_FDMOD);

	if(husb->Init.usb_otg_mode == USB_OTG_MODE_HOST){
		gusbcfg |= USB_OTG_GUSBCFG_FHMOD;
	}
	else if(husb->Init.usb_otg_mode == USB_OTG_MODE_DEVICE){
		gusbcfg |= USB_OTG_GUSBCFG_FDMOD;
	}
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GUSBCFG), gusbcfg);
	dly_tsk(50);
	return E_OK;
}

/*
 *  USBコアの初期化
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_coreinit(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	volatile uint32_t tick = 0;

	/*
	 *  コアリセット
	 */

	/*
	 *  AHBマスターのIDLEステート待ち
	 */
	do{
		if(++tick > RESET_TIMEOUT)
			return E_TMOUT;
		sil_dly_nse(10*1000);
	}
	while ((sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRSTCTL)) & USB_OTG_GRSTCTL_AHBIDL) == 0);

	/*
	 *  コア、ソフトリセット要求
	 */
	tick = 0;
	sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRSTCTL), USB_OTG_GRSTCTL_CSRST);
	do{
		if(++tick > RESET_TIMEOUT)
			return E_TMOUT;
		sil_dly_nse(10*1000);
	}
	while ((sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRSTCTL)) & USB_OTG_GRSTCTL_CSRST) != 0);

	/*
	 *  DMAモード設定
	 */
	if(husb->Init.dma_enable == 1){
		sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GAHBCFG), USB_OTG_GAHBCFG_DMAEN);
	}
	return E_OK;
}

/*
 *  USBグローバル割込み許可
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_enableglobalint(USB_OTG_Handle_t *husb)
{
	sil_orw_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GAHBCFG), USB_OTG_GAHBCFG_GINT);
	return E_OK;
}

/*
 *  USBグローバル割込み禁止
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_disableglobalint(USB_OTG_Handle_t *husb)
{
	sil_andw_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GAHBCFG), USB_OTG_GAHBCFG_GINT);
	return E_OK;
}

/*
 *  送信FIFOのフラッシュ
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     正常終了：E_OK、タイムアウト：E_TMOUT
 */
ER
usbo_flushTxFifo(USB_OTG_Handle_t *husb, uint32_t num)
{
	uint32_t base = husb->base;
	uint32_t tick = 0;

	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRSTCTL), (USB_OTG_GRSTCTL_TXFFLSH |(uint32_t)(num << 6)));
	do{
		if(++tick > RESET_TIMEOUT)
			return E_TMOUT;
		sil_dly_nse(10*1000);
	}while ((sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRSTCTL)) & USB_OTG_GRSTCTL_TXFFLSH) != 0);
	return E_OK;
}

/*
 *  受信FIFOのフラッシュ
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     正常終了：E_OK、タイムアウト：E_TMOUT
 */
ER
usbo_flushRxFifo(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t tick = 0;

	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRSTCTL), USB_OTG_GRSTCTL_RXFFLSH);
	do{
		if(++tick > RESET_TIMEOUT)
			return E_TMOUT;
		sil_dly_nse(10*1000);
	}while((sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRSTCTL)) & USB_OTG_GRSTCTL_RXFFLSH) != 0);
	return E_OK;
}

/*
 *  FIFOの初期化
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_initFiFo(USB_OTG_Handle_t *husb)
{
	unsigned int ep;
	unsigned int addr;
	uint32_t base = husb->base;
	uint32_t val;

	/*
	 *  RX/NPTX FIFOサイズ設定
	 */
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRXFSIZ), husb->pcd_np_g_tx_fifo_sz);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_DIEPTXF0_HNPTXFSIZ),
			(husb->pcd_np_g_tx_fifo_sz<<16) | husb->pcd_rx_fifo_sz);

	/*
	 *  すべてのTX FIFO設定
	 */
	addr = husb->pcd_rx_fifo_sz + husb->pcd_np_g_tx_fifo_sz;
	for(ep = 1; ep < MAX_EPS_CHANNELS; ep++) {
		if(husb->pcd_tx_fifo_sz[ep] == 0){
			sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_DIEPTXF+(ep-1)*4), 0);
			continue;
		}
		val = addr | husb->pcd_tx_fifo_sz[ep] << 16;
		if((addr + husb->pcd_tx_fifo_sz[ep]) > husb->fifo_mem)
			syslog_1(LOG_ERROR, "insufficient fifo memory(%d)", ep);
		addr += husb->pcd_tx_fifo_sz[ep];
		sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_DIEPTXF+(ep-1)*4), val);
	}

	/*
	 * FIFOフラッシュ処理
	 */
	usbo_flushTxFifo(husb, 0x10);
	usbo_flushRxFifo(husb);
	return E_OK;
}

/*******************************************************************************
                       USB HOST関数
*******************************************************************************/
/*
 *  FSLSPClkSel設定
 *  parameter1 husb  USBハンドラへのポインタ
 */
static void
usbo_initFSLSPClkSel(USB_OTG_Handle_t *husb)
{
	uint32_t hcfg, val;

	if(husb->phy_type == PHY_TYPE_PARAM_FS)
		val = USB_OTG_HCFG_FSLSPCS_0;	/* Full speed PHY 48000 */
	else
		val = 0;	/* High speed PHY running at full speed or high speed */
	hcfg = sil_rew_mem((uint32_t *)(USBH_BASE(husb->base)+TOFF_USBH_HCFG));
	hcfg &= ~USB_OTG_HCFG_FSLSPCS;
	hcfg |= val;
	sil_wrw_mem((uint32_t *)(USBH_BASE(husb->base)+TOFF_USBH_HCFG), hcfg);
}

/*
 *  HOST FS-PHY初期化
 */
static ER
usbo_fs_phy_init(USB_OTG_Handle_t *husb)
{
	uint32_t usbcfg;
	ER ercd = E_OK;

	usbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GUSBCFG));
	if((usbcfg & USB_OTG_GUSBCFG_PHYSEL) == 0){
		usbcfg |= USB_OTG_GUSBCFG_PHYSEL;
		sil_wrw_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GUSBCFG), usbcfg);

		/*
		 *  PHY選択をリセット
		 */
		ercd = usbo_coreinit(husb);
		if(ercd != E_OK){
			syslog_0(LOG_ERROR,	"usbo_fs_phy_init: Reset failed, aborting !");
			return ercd;
		}
		usbo_setcurrentmode(husb);
	}

	/*
	 * ホスト固定なら、FS-PHY:HCFG.FSLSPclkSelを48Mhzに設定
	 */
	if(get_usbmode(husb->base) == USB_OTG_MODE_HOST)
		usbo_initFSLSPClkSel(husb);
	return ercd;
}

/*
 *  HOST HS-PHY初期化
 */
static ER
usbo_hs_phy_init(USB_OTG_Handle_t *husb)
{
	uint32_t usbcfg, usbcfg_old;
	ER ercd = E_OK;

	usbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GUSBCFG));
	usbcfg_old = usbcfg;

	/*
	 *  HS PHYタイプによるUSBCFGレジスタ設定
	 */
	switch (husb->phy_type) {
	case PHY_TYPE_PARAM_ULPI:
		/* ULPI interface */
		syslog_0(LOG_NOTICE, "HS ULPI PHY selected");
		usbcfg |= USB_OTG_GUSBCFG_ULPIUTSEL;
		usbcfg &= ~(GUSBCFG_PHYIF16 | USB_OTG_GUSBCFG_DDRSEL);
		break;
	case PHY_TYPE_PARAM_UTMI:
		/* UTMI+ interface */
		syslog_0(LOG_NOTICE, "HS UTMI+ PHY selected");
		usbcfg &= ~(USB_OTG_GUSBCFG_ULPIUTSEL | GUSBCFG_PHYIF16);
		if (husb->phy_utmi_width == 16)
			usbcfg |= GUSBCFG_PHYIF16;
		break;
	default:
		syslog_0(LOG_ERROR, "FS PHY selected at HS!");
		break;
	}

	if(usbcfg != usbcfg_old){
		sil_wrw_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GUSBCFG), usbcfg);

		/*
		 *  PHY選択をリセット
		 */
		ercd = usbo_coreinit(husb);
		if(ercd != E_OK){
			syslog_0(LOG_ERROR, "usbo_hs_phy_init: Reset failed, aborting !");
			return ercd;
		}
		usbo_setcurrentmode(husb);
	}
	return ercd;
}

/*
 *  HOST PHY初期化
 */
static ER
usbo_phy_init(USB_OTG_Handle_t *husb)
{
	ER ercd = E_OK;

	if(husb->Init.speed > USB_SPEED_HIGH_IN_FULL &&
	    husb->phy_type == PHY_TYPE_PARAM_FS) {
		/*
		 *  FS PHY設定
		 */
		ercd = usbo_fs_phy_init(husb);
	} else {
		/*
		 *  HS PHY設定
		 */
		ercd = usbo_hs_phy_init(husb);
	}
	return ercd;
}

/*
 *  HCDモードコモン割込み設定
 */
static void
usbo_enable_common_interrupts(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t intmsk;

	/*
	 *  ペンディング割込みクリア
	 */
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGINT), 0xFFFFFFFF);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTSTS), 0xFFFFFFFF);

	/*
	 *  GINTMSK基本設定
	 */
	intmsk = USB_OTG_GINTMSK_MMISM | USB_OTG_GINTMSK_OTGINT;
	if (husb->Init.dma_enable <= 0)
		intmsk |= USB_OTG_GINTMSK_RXFLVLM;
	intmsk |= USB_OTG_GINTMSK_WUIM | USB_OTG_GINTMSK_USBSUSPM |
		  USB_OTG_GINTMSK_SRQIM | USB_OTG_GINTMSK_CIDSCHGM;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), intmsk);
}

/*
 *  HCDモード割込み有効化
 */
static void
usbo_enable_host_interrupts(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t intmsk;

	/*
	 *  すべての割込みマスク
	 */
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), 0);
	sil_wrw_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HAINTMSK), 0);

	/*
	 *  基本割込み有効化
	 */
	usbo_enable_common_interrupts(husb);

	/*
	 *  HCDモード割込み有効化
	 */
	intmsk = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK));
	intmsk |= USB_OTG_GINTMSK_DISCINT | USB_OTG_GINTMSK_PRTIM | USB_OTG_GINTMSK_HCIM;
	intmsk |= USB_OTG_GINTMSK_PXFRM_IISOOXFRM | USB_OTG_GINTMSK_SOFM;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), intmsk);
	syslog_1(LOG_NOTICE, "## usbo_enable_host_interrupts[%08x] ##", intmsk);
}

/*
 *  HCDモード割込み無効化
 */
static void
usbo_disable_host_interrupts(USB_OTG_Handle_t *husb)
{
	uint32_t intmsk = sil_rew_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GINTMSK));

	/*
	 *  HCDモード割込み無効化
	 */
	intmsk &= ~(USB_OTG_GINTMSK_SOFM | USB_OTG_GINTMSK_PRTIM | USB_OTG_GINTMSK_HCIM |
		    USB_OTG_GINTMSK_PTXFEM | USB_OTG_GINTMSK_NPTXFEM | USB_OTG_GINTMSK_DISCINT);
	intmsk &= ~(USB_OTG_GINTMSK_PXFRM_IISOOXFRM);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GINTMSK), intmsk);
	syslog_1(LOG_NOTICE, "## usbo_disable_host_interrupts[%08x] ##", intmsk);
}

/*
 *  HOST-FIFO設定
 */
static void
usbo_config_host_fifos(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t nptxfsiz, hptxfsiz, grxfsiz;

	if (!husb->enable_dynamic_fifo)
		return;

	/* Rx FIFO */
	grxfsiz = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRXFSIZ));
	grxfsiz &= ~USB_OTG_GRXFSIZ_RXFD;
	grxfsiz |= husb->host_rx_fifo_size & USB_OTG_GRXFSIZ_RXFD;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRXFSIZ), grxfsiz);

	/* Non-periodic Tx FIFO */
	nptxfsiz = husb->host_nperio_tx_fifo_size << 16;
	nptxfsiz |= husb->host_rx_fifo_size & USB_OTG_NPTXFSA;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_DIEPTXF0_HNPTXFSIZ), nptxfsiz);

	/* Periodic Tx FIFO */
	hptxfsiz = husb->host_perio_tx_fifo_size << 16;
	hptxfsiz |= (husb->host_rx_fifo_size +
		     husb->host_nperio_tx_fifo_size) & USB_OTG_HPTXFSIZ_PTXSA;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_HPTXFSIZ), hptxfsiz);
}

/*
 *  OTGホストモードスタート
 */
static void
usbo_hcd_start(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t hcchar;
	int i;

	enter_spinlock(husb->Init.spnlockid);
	/* Restart the Phy Clock resume発生ならそこで対応？ */
	sil_wrw_mem((uint32_t *)(USBPGC_BASE(base)), 0);

	/*
	 *  PHYクロック設定
	 */
	usbo_initFSLSPClkSel(husb);

	/*
	 *  FS/LSモード設定
	 */
	if(husb->Init.speed == USB_SPEED_FULL){
		sil_orw_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HCFG), USB_OTG_HCFG_FSLSS);
	}
	else{
		sil_andw_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HCFG), USB_OTG_HCFG_FSLSS);
	}

	/*
	 *  ダイナミック・ローディング機能を有効化
	 */
	sil_orw_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HFIR), USB_OTG_HFIR_HFIRRLDCTRL);

	/*
	 *  HOST FIFO設定
	 */
	usbo_config_host_fifos(husb);

	/*
	 *  ホストがセットしたHNP Enableをクリア
	 */
	sil_andw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGCTL), USB_OTG_GOTGCTL_HSHNPEN);

	/*
	 *  FIFOフラッシュ
	 */
	usbo_flushTxFifo(husb, 0x10 /* all TX FIFOs */);
	usbo_flushRxFifo(husb);

	/*
	 *  ホストがセットしたHNP Enableをクリア
	 */
	sil_andw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGCTL), USB_OTG_GOTGCTL_HSHNPEN);

	/*
	 *  リクエスト情報をフラッシュ
	 */
	for(i = 0; i < husb->Init.host_channels ; i++){
		hcchar = sil_rew_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCCHAR));
		hcchar &= ~USB_OTG_HCCHAR_CHENA;
		hcchar |= USB_OTG_HCCHAR_CHDIS;
		hcchar &= ~USB_OTG_HCCHAR_EPDIR;
		sil_wrw_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCCHAR), hcchar);
	}

	/*
	 *  すべてチャネルをHALT
	 */
	for(i = 0 ; i < husb->Init.host_channels ; i++){
		volatile int count = 0;

		husb->hc[i].state = USBO_HC_IDLE;
		hcchar = sil_rew_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCCHAR));
		hcchar |= USB_OTG_HCCHAR_CHENA | USB_OTG_HCCHAR_CHDIS;
		hcchar &= ~USB_OTG_HCCHAR_EPDIR;
		sil_wrw_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCCHAR), hcchar);
		do {
			hcchar = sil_rew_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCCHAR));
			if (++count > 1000) {
				husb->hc[i].state = USBO_HC_FAIL;
				syslog_1(LOG_ERROR, "Unable to clear enable on channel %d", i);
				break;
			}
			sil_dly_nse(1000);
		} while (hcchar & USB_OTG_HCCHAR_CHENA);
	}

	/*
	 *  VBUS電源オン
	 */
	syslog_1(LOG_NOTICE, "Init: Port Power? op_state=%d", husb->op_state);
	if(husb->op_state == OTG_STATE_A_HOST) {
		uint32_t hprt0 = sil_rew_mem((uint32_t*)(USBPRT_BASE(base)));
		hprt0 &= ~(USB_OTG_HPRT_PENA | USB_OTG_HPRT_PCDET | USB_OTG_HPRT_PENCHNG | USB_OTG_HPRT_POCCHNG);

		syslog_1(LOG_NOTICE, "Init: Power Port (%d)",
			!!(hprt0 & USB_OTG_HPRT_PPWR));
		if (!(hprt0 & USB_OTG_HPRT_PPWR)) {
			hprt0 |= USB_OTG_HPRT_PPWR;
			sil_wrw_mem((uint32_t*)(USBPRT_BASE(base)), hprt0);
		}
	}
	usbo_enable_host_interrupts(husb);
	release_spinlock(husb->Init.spnlockid);
}

/*
 *  USB HOST初期化
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_hostinit(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t i;
	uint32_t usbcfg, otgctl, ahbcfg;
	ER ercd = E_OK;

	enter_spinlock(husb->Init.spnlockid);
	/*
	 *  グローバル割込みを禁止
	 */
	usbo_disableglobalint(husb);

	/*
	 *  PHYを初期化
	 */
	ercd = usbo_phy_init(husb);
	if(ercd != E_OK){
		release_spinlock(husb->Init.spnlockid);
		syslog_1(LOG_ERROR, "usbo_hostinit phy error (%d) !", ercd);
		return ercd;
	}

	/*
	 *  GAHBCFGレジスタ設定
	 */
	ahbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GAHBCFG));
	switch (husb->hw_params.arch) {
	case GHWCFG2_EXT_DMA_ARCH:
		release_spinlock(husb->Init.spnlockid);
		syslog_0(LOG_ERROR, "External DMA Mode not supported");
		return E_NOSPT;

	case GHWCFG2_INT_DMA_ARCH:
		ahbcfg &= GAHBCFG_CTRL_MASK;
		ahbcfg |= GAHBCFG_HBSTLEN_INCR4;
		break;

	case GHWCFG2_SLAVE_ONLY_ARCH:
	default:
		break;
	}

	if(husb->Init.dma_enable > 0)
		ahbcfg |= USB_OTG_GAHBCFG_DMAEN;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GAHBCFG), ahbcfg);

	/*
	 *  GUSBCFGレジスタ設定
	 */
	usbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GUSBCFG));
	/* Not Support ULPI External VBUS */
	usbcfg &= ~USB_OTG_GUSBCFG_ULPIEVBUSD;

	switch (husb->hw_params.op_mode) {
	case GHWCFG2_OP_MODE_HNP_SRP_CAPABLE:
		if (husb->otg_cap == DWC2_CAP_PARAM_HNP_SRP_CAPABLE)
			usbcfg |= USB_OTG_GUSBCFG_HNPCAP;
		if (husb->otg_cap != DWC2_CAP_PARAM_NO_HNP_SRP_CAPABLE)
			usbcfg |= USB_OTG_GUSBCFG_SRPCAP;
		break;

	case GHWCFG2_OP_MODE_SRP_ONLY_CAPABLE:
	case GHWCFG2_OP_MODE_SRP_CAPABLE_DEVICE:
	case GHWCFG2_OP_MODE_SRP_CAPABLE_HOST:
		if (husb->otg_cap != DWC2_CAP_PARAM_NO_HNP_SRP_CAPABLE)
			usbcfg |= USB_OTG_GUSBCFG_SRPCAP;
		break;

	case GHWCFG2_OP_MODE_NO_HNP_SRP_CAPABLE:
	case GHWCFG2_OP_MODE_NO_SRP_CAPABLE_DEVICE:
	case GHWCFG2_OP_MODE_NO_SRP_CAPABLE_HOST:
	default:
		break;
	}
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(husb->base)+TOFF_USBO_GUSBCFG), usbcfg);

	/*
	 *  GOTGCTLレジスタ設定
	 */
	otgctl = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGCTL));
	otgctl &= ~USB_OTG_GOTGCTL_OTGVER;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGCTL), otgctl);

	/*
	 *  コモン割込み設定
	 */
	usbo_enable_common_interrupts(husb);

	/*
	 * Do device or host initialization based on mode during PCD and
	 * HCD initialization
	 */
	if(husb->Init.usb_otg_mode == USB_OTG_MODE_DRD){
		syslog_0(LOG_NOTICE, "OTG Mode");
		husb->op_state = OTG_STATE_A_WAIT_VRISE;
	}
	else if(get_usbmode(husb->base) == USB_OTG_MODE_HOST){
		syslog_0(LOG_NOTICE, "Host Mode");
		husb->op_state = OTG_STATE_A_HOST;
	}
	else{
		syslog_0(LOG_NOTICE, "Device Mode");
		husb->op_state = OTG_STATE_B_PERIPHERAL;
	}

	/*
	 * チャネル割込みクリアとマスク
	 */
	for (i = 0 ; i < husb->Init.host_channels; i++){
		sil_wrw_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCINT), 0xFFFFFFFFU);
		sil_wrw_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCINTMSK), 0U);
	}

	/*
	 *  L3ステート設定
	 */
	husb->lx_state = DWC2_L3;

	/*
	 *  HOST VBUS設定
	 */
	usbo_drivevbus(husb, 1);
	release_spinlock(husb->Init.spnlockid);
	dly_tsk(200);
	return E_OK;
}

/*
 *  HOSTコア開始要求
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_starthost(USB_OTG_Handle_t *husb)
{
	syslog_2(LOG_NOTICE, "DWC OTG HCD START op_state(%d) (%d)", husb->op_state, get_usbmode(husb->base));

	if(husb->op_state < OTG_STATE_A_HOST)
		return E_OK;

	husb->lx_state = DWC2_L0;
	if(get_usbmode(husb->base) == USB_OTG_MODE_HOST){
		/* DWCコア ホスト・モード開始 */
		usbo_hcd_start(husb);
		dly_tsk(25);	/* コア設定後待ち(割込み中の場合あり) */
	}
	else{
		/* DWCコア デバイス・モード開始 */
		usbo_pcd_control(husb, 1);
	}
	return E_OK;
}

/*
 *  HOSTコア停止要求
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_stophost(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t intr, hcchar, hcintmsk;
	int i;

	enter_spinlock(husb->Init.spnlockid);
	usbo_disableglobalint(husb);

	/*
	 *  USBホスト用の割込みクリア
	 */
	sil_wrw_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HAINT), 0xFFFFFFFFU);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTSTS), 0xFFFFFFFFU);

	/*
	 * Shutdown any transfers in process by clearing the Tx FIFO Empty
	 * interrupt mask and status bits and disabling subsequent host
	 * channel interrupts.
	 */
	intr = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK));
	intr &= ~(USB_OTG_GINTMSK_NPTXFEM | USB_OTG_GINTMSK_PTXFEM | USB_OTG_GINTMSK_HCIM);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), intr);
	intr = USB_OTG_GINTSTS_NPTXFE | USB_OTG_GINTSTS_PTXFE | USB_OTG_GINTSTS_HCINT;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTSTS), intr);

	/*
	 * Turn off the vbus power only if the core has transitioned to device
	 * mode. If still in host mode, need to keep power on to detect a
	 * reconnection.
	 */
	if(get_usbmode(base) == USB_OTG_MODE_HOST){
		/* Clean up any host channels that were in use */
		if (husb->Init.dma_enable == 0) {
			/* Flush out any channel requests in slave mode */
			for(i = 0 ; i < husb->Init.host_channels ; i++){
				hcchar = sil_rew_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCCHAR));
				if (hcchar & USB_OTG_HCCHAR_CHENA) {
					hcchar &= ~(USB_OTG_HCCHAR_CHENA | USB_OTG_HCCHAR_EPDIR);
					hcchar |= USB_OTG_HCCHAR_CHDIS;
					sil_wrw_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCCHAR), hcchar);
				}
			}
		}

		for(i = 0 ; i < husb->Init.host_channels ; i++){
			hcchar = sil_rew_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCCHAR));
			if (hcchar & USB_OTG_HCCHAR_CHENA) {
				if(husb->Init.dma_enable == 0)
					hcchar &= ~(USB_OTG_HCCHAR_CHENA | USB_OTG_HCCHAR_EPDIR);
				/* Halt the channel */
				hcchar |= USB_OTG_HCCHAR_CHDIS;
				sil_wrw_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCCHAR), hcchar);
			}

			sil_wrw_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCINTMSK), 0U);
			hcintmsk = 0xffffffff;
			hcintmsk &= ~HCINTMSK_RESERVED14_31;
			sil_wrw_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCINT), hcintmsk);
		}
	}

	usbo_disable_host_interrupts(husb);
	/* Turn off the vbus power */
	syslog_0(LOG_NOTICE, "PortPower off");
	sil_wrw_mem((uint32_t*)(USBPRT_BASE(base)), 0);

	if(husb->Init.usb_otg_mode == USB_OTG_MODE_DRD){
		release_spinlock(husb->Init.spnlockid);
#if 1	/* ROI DEBUG DEBUG DEBUG */
		syslog_0(LOG_NOTICE, "## REINIT ##");
		usbo_devinit(husb);
		dly_tsk(25);
		usbo_devconnect(husb);
		usbo_hostinit(husb);
#endif	/* ROI DEBUG DEBUG DEBUG */
		husb->op_state = OTG_STATE_A_WAIT_VRISE;
	}
	else{
		husb->lx_state = DWC2_L3;
		release_spinlock(husb->Init.spnlockid);
	}
	dly_tsk(25);
	return E_OK;
}

/*
 *  HOSTチャネルの初期化
 *  parameter1 husb   USBハンドラへのポインタ
 *  parameter2 ch_num チャネル番号(1-15)
 *  parameter3 epnum  エンドポイント番号(1-15)
 *  return     常にE_OK
 */
ER
usbo_hc_init(USB_OTG_Handle_t *husb, uint8_t ch_num, uint8_t epnum)
{
	uint32_t base = husb->base;
	uint8_t  address = husb->hc[ch_num].dev_addr;
	uint8_t  speed   = husb->hc[ch_num].speed;
	uint8_t  ep_type = husb->hc[ch_num].ep_type;
	uint16_t mps     = husb->hc[ch_num].max_packet;
	uint32_t hcint   = 0;

	/*
	 *  チャネルの割込みクリア
	 */
	sil_wrw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCINT), 0xFFFFFFFFU);

	if(husb->Init.dma_enable == 1){
		hcint = USB_OTG_HCINTMSK_CHHM;
		/*
		 * For Descriptor DMA mode core halts the channel on AHB error.
		 * Interrupt is not required.
		 */
		if(ep_type == EP_TYPE_ISOC)
			hcint |= USB_OTG_HCINTMSK_XFRCM;
	}
	else{
		/*
		 *  エンドポイント型に従う初期化
		 */
		switch (ep_type) {
		case EP_TYPE_CTRL:
		case EP_TYPE_BULK:
			hcint = USB_OTG_HCINTMSK_XFRCM  | USB_OTG_HCINTMSK_STALLM |\
	                USB_OTG_HCINTMSK_TXERRM | USB_OTG_HCINTMSK_DTERRM |\
	                USB_OTG_HCINTMSK_AHBERR | USB_OTG_HCINTMSK_NAKM;

			if(epnum & 0x80U)
				hcint |= USB_OTG_HCINTMSK_BBERRM;
			else
				hcint |= USB_OTG_HCINTMSK_NYETM | USB_OTG_HCINTMSK_ACKM;
			break;

		case EP_TYPE_INTR:
			hcint = USB_OTG_HCINTMSK_XFRCM  | USB_OTG_HCINTMSK_STALLM |\
	                USB_OTG_HCINTMSK_TXERRM | USB_OTG_HCINTMSK_DTERRM |\
	                USB_OTG_HCINTMSK_NAKM   | USB_OTG_HCINTMSK_AHBERR | USB_OTG_HCINTMSK_FRMORM;

			if(epnum & 0x80)
				hcint |= USB_OTG_HCINTMSK_BBERRM;
			break;
		case EP_TYPE_ISOC:
				hcint = USB_OTG_HCINTMSK_XFRCM  | USB_OTG_HCINTMSK_ACKM |\
	                    USB_OTG_HCINTMSK_AHBERR | USB_OTG_HCINTMSK_FRMORM;

			if(epnum & 0x80)
				hcint |= USB_OTG_HCINTMSK_TXERRM | USB_OTG_HCINTMSK_BBERRM;
			break;
		}
	}

	sil_wrw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCINTMSK), hcint);

	/*
	 *  HOSTチャネル割込みマスク解除
	 */
	sil_orw_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HAINTMSK), (1 << ch_num));

	/*
	 *  HOSTチャネル割込み許可
	 */
	sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), USB_OTG_GINTMSK_HCIM);

	/*
	 *  HCCHARレジスタ設定
	 */
	sil_wrw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR), (((address << 22U) & USB_OTG_HCCHAR_DAD)  |\
                             (((epnum & 0x7FU)<< 11U) & USB_OTG_HCCHAR_EPNUM)|\
                             ((((epnum & 0x80U) == 0x80U)<< 15U) & USB_OTG_HCCHAR_EPDIR)|\
                             (((speed == USB_SPEED_LOW)<< 17U) & USB_OTG_HCCHAR_LSDEV)|\
                             ((ep_type << 18U) & USB_OTG_HCCHAR_EPTYP)|\
                             (mps & USB_OTG_HCCHAR_MPSIZ)));

	if(ep_type == EP_TYPE_INTR){
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_ODDFRM);
	}
	return E_OK; 
}

/*
 *  HOSTコアスピードを取り出す
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     speed USB_SPEED_HIGH(High speed)
 *                   USB_SPEED_FULL(Full speed)
 *                   USB_SPEED_LOW(Low speed)
 */
uint32_t
usbo_gethostspeed(USB_OTG_Handle_t *husb)
{
	volatile uint32_t hprt0;

	hprt0 = sil_rew_mem((uint32_t*)(USBPRT_BASE(husb->base)));
	return ((hprt0 & USB_OTG_HPRT_PSPD) >> 17);
}

/*
 *  HOST DMA実行終了待ち
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 ch_num チャネル番号(1-15)
 */
void
usbo_dma_wait(USB_OTG_Handle_t *husb, uint8_t ch_num)
{
	USB_OTG_HCTypeDef *hc = &husb->hc[ch_num];
	uint32_t dly_count = 0;

	while((hc->pend_int != 0 || hc->chg_urb != 0) && dly_count < 5){
		dly_tsk(1);
		dly_count++;
	}
}

/*
 *  HOSTチャネル送信要求
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 ch_num チャネル番号(1-15)
 *  return     常にE_OK
 */
ER
usbo_hc_startxfer(USB_OTG_Handle_t *husb, uint8_t ch_num)
{
	USB_OTG_HCTypeDef *hc = &husb->hc[ch_num];
	uint32_t base = husb->base;
	uint8_t  is_oddframe = 0U; 
	uint16_t len_words = 0U;   
	uint16_t num_packets = 0U;
	uint16_t max_hc_pkt_count = 256U;

	/*
	 *  DMA転送エラーケース割込み追加
	 */
	if(husb->Init.dma_enable == 1 && hc->err_count > 0 &&
			hc->ep_type != EP_TYPE_ISOC){
		uint32_t hcintmsk = USB_OTG_HCINTMSK_ACKM;
		if(hc->ep_is_in){
			hcintmsk |= USB_OTG_HCINTMSK_DTERRM;
			if(hc->ep_type != EP_TYPE_INTR)
				hcintmsk |= USB_OTG_HCINTMSK_NAKM;
		}
		hc->pend_msk |= hcintmsk;
		dly_tsk(1);
	}

	if(/*(base != TADR_USB_OTGFS_BASE) &&*/ (hc->speed == USB_SPEED_HIGH)){
		if((husb->Init.dma_enable == 0) && (hc->do_ping == 1)){
			/*
			 * Do Ping
			 */
			sil_wrw_mem((uint32_t *)(USBHC_BASE(base, hc->ch_num)+TOFF_USBHC_HCTSIZ), ((1 << 19) & USB_OTG_HCTSIZ_PKTCNT) | USB_OTG_HCTSIZ_DOPING);
			sil_modw_mem((uint32_t *)(USBHC_BASE(base, hc->ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHDIS, USB_OTG_HCCHAR_CHENA);
			return E_OK;
		}
		else if(husb->Init.dma_enable == 1){
			sil_andw_mem((uint32_t *)(USBHC_BASE(base, hc->ch_num)+TOFF_USBHC_HCINTMSK), (USB_OTG_HCINTMSK_NYETM | USB_OTG_HCINTMSK_ACKM));
			hc->do_ping = 0U;
		}
	}

	/*
	 *  パケット数、転送長の計算
	 */
	if(hc->xfer_len > 0){
		num_packets = (hc->xfer_len + hc->max_packet - 1U) / hc->max_packet;

		if(num_packets > max_hc_pkt_count){
			num_packets = max_hc_pkt_count;
			hc->xfer_len = num_packets * hc->max_packet;
		}
	}
	else{
		num_packets = 1;
	}
#if 0	/* ROI DEBUG */
	if(hc->ep_is_in){
#else	/* ROI DEBUG */
	if(hc->ep_is_in && hc->xfer_buff != 0){
#endif	/* ROI DEBUG */
		hc->xfer_len = num_packets * hc->max_packet;
	}

	/*
	 *  HCTSIZレジスタ設定
	 */
	sil_wrw_mem((uint32_t *)(USBHC_BASE(base, hc->ch_num)+TOFF_USBHC_HCTSIZ), (((hc->xfer_len) & USB_OTG_HCTSIZ_XFRSIZ)) |\
		((num_packets << 19U) & USB_OTG_HCTSIZ_PKTCNT) | (((hc->data_pid) << 29U) & USB_OTG_HCTSIZ_DPID));

#if 0	/* ROI DEBUG */
	syslog_3(LOG_NOTICE, "## usbo_hc_startxfer ch(%d) [%08x] (%d) ##", ch_num, hc->xfer_buff, hc->xfer_len);
#endif	/* ROI DEBUG */
	if(husb->Init.dma_enable){
		flushinvalidatedcache_by_addr(((uint8_t *)hc->xfer_buff), hc->xfer_len);
		sil_wrw_mem((uint32_t *)(USBHC_BASE(base, hc->ch_num)+TOFF_USBHC_HCDMA), (uint32_t)hc->xfer_buff);
	}

	/*
	 *  EVEN/ODDパケット設定
	 */
	is_oddframe = (sil_rew_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HFNUM)) & 0x01U) ? 0U : 1U;
	sil_andw_mem((uint32_t *)(USBHC_BASE(base, hc->ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_ODDFRM);
	sil_orw_mem((uint32_t *)(USBHC_BASE(base, hc->ch_num)+TOFF_USBHC_HCCHAR), (is_oddframe << 29U));

	/*
	 *  HOSTチャネルを有効化
	 */
	sil_modw_mem((uint32_t *)(USBHC_BASE(base, hc->ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHDIS, USB_OTG_HCCHAR_CHENA);

	if(husb->Init.dma_enable == 0){	/* none DMAモード */
		if((hc->ep_is_in == 0U) && (hc->xfer_len > 0U)){
			switch(hc->ep_type) {
			case EP_TYPE_CTRL:	/* NON PERIODIC転送 */
			case EP_TYPE_BULK:
				len_words = (hc->xfer_len + 3) / 4;
				/*
				 *  FIFO長を超える場合、NPTXFEMPTY割込みを設定
				 */
				if(len_words > (sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_HNPTXSTS)) & 0xFFFFU)){
					sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), USB_OTG_GINTMSK_NPTXFEM);
				}
				break;
			case EP_TYPE_INTR:	/* PERIODIC 転送 */
			case EP_TYPE_ISOC:
				len_words = (hc->xfer_len + 3) / 4;
				/*
				 *  FIFO長を超える場合、PTXFEMPTY割込みを設定
				 */
				if(len_words > (sil_rew_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HPTXSTS)) & 0xFFFFU)){ /* split the transfer */
					sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), USB_OTG_GINTMSK_PTXFEM);
				}
				break;
			default:
				break;
			}

			/*
			 *  パケット書き込み
			 */
			usbo_writePacket(base, hc->xfer_buff, hc->ch_num, hc->xfer_len, 0);
		}
	}
#if 1	/* ROI DEBUG */
	if(hc->ep_is_in && hc->xfer_len > 0)
		dly_tsk(1);
#endif	/* ROI DEBUG */
	return E_OK;
}

/*
 *  HOSTチャネルをHALTする
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 チャネル番号(1-15)
 *  return     常にE_OK
 */
ER
usbo_hc_halt(USB_OTG_Handle_t *husb, uint8_t ch_num)
{
	uint32_t base = husb->base;
	uint32_t tick = 0;

	/* HCCHARコントロール・バルクモード */
	if((((sil_rew_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR)) & USB_OTG_HCCHAR_EPTYP) >> 18) == HCCHAR_CTRL)
			|| ((((sil_rew_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR)) & USB_OTG_HCCHAR_EPTYP) >> 18) == HCCHAR_BULK))){
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHDIS);
		if((sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_HNPTXSTS)) & 0xFF0000U) == 0){
			sil_andw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHENA);
			sil_orw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHENA);
			do{
				if(++tick > HALT_TIMEOUT)
					break;
				sil_dly_nse(10*1000);
			}while((sil_rew_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR)) & USB_OTG_HCCHAR_CHENA) != 0);
		}
		else{
			sil_orw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHENA);
		}
	}
	else{	/* その他 */
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHDIS);
#if 1	/* ROI DEBUG */
//			if(husb->Init.dma_enable == 1)
//				return E_OK;
#endif	/* ROI DEBUG */
		if((sil_rew_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HPTXSTS)) & 0xFFFF) == 0){
			sil_andw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHENA);
			sil_orw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHENA);
			do{
				if(++tick > HALT_TIMEOUT) 
					break;
				sil_dly_nse(10*1000);
			}while((sil_rew_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR)) & USB_OTG_HCCHAR_CHENA) != 0);
		}
		else{
			sil_orw_mem((uint32_t *)(USBHC_BASE(base, ch_num)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHENA);
		}
	}
	return E_OK;
}

/*
 *  USB OTG HOSTポートリセット
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_resetport(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	volatile uint32_t hprt0;

	hprt0 = sil_rew_mem((uint32_t*)(USBPRT_BASE(base)));
	hprt0 &= ~(USB_OTG_HPRT_PENA | USB_OTG_HPRT_PCDET | USB_OTG_HPRT_PENCHNG | USB_OTG_HPRT_POCCHNG);

	sil_wrw_mem((uint32_t*)(USBPRT_BASE(base)), (USB_OTG_HPRT_PRST | hprt0));
	dly_tsk (10);				/* See Note #1 */
	sil_wrw_mem((uint32_t*)(USBPRT_BASE(base)), (~(USB_OTG_HPRT_PRST) & hprt0));
	return E_OK;
}

/*
 *  USB HOST VBUSの有効、無効設定
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 state 1:VBUS-ACTIVE 0:VBUS-INACTIVE
 *  return     常にE_OK
 */
ER
usbo_drivevbus(USB_OTG_Handle_t *husb, uint8_t state)
{
	uint32_t base = husb->base;
	volatile uint32_t hprt0;

	hprt0 = sil_rew_mem((uint32_t*)(USBPRT_BASE(base)));
	hprt0 &= ~(USB_OTG_HPRT_PENA | USB_OTG_HPRT_PCDET | USB_OTG_HPRT_PENCHNG | USB_OTG_HPRT_POCCHNG);

	if(((hprt0 & USB_OTG_HPRT_PPWR) == 0) && (state == 1)){
		sil_wrw_mem((uint32_t*)(USBPRT_BASE(base)), (USB_OTG_HPRT_PPWR | hprt0));
	}
	if(((hprt0 & USB_OTG_HPRT_PPWR) != 0) && (state == 0)){
		sil_wrw_mem((uint32_t*)(USBPRT_BASE(base)), ((~USB_OTG_HPRT_PPWR) & hprt0));
	}
	return E_OK; 
}

/*
 *  HOST EXTERNAL VBUSの有効、無効設定
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 state 0:VBUS-ACTIVE 1:VBUS-INACTIVE
 */
void
usbo_driveextvbus(USB_OTG_Handle_t *husb, uint8_t state)
{
}

/*
 *  DMA転送終了割込み処理
 */
static void
usbo_hc_xfercomp_intr(USB_OTG_Handle_t *husb, USB_OTG_HCTypeDef *hc, int chnum, bool_t in)
{
	uint32_t base = husb->base;
	if(in){
		hc->xfer_count = hc->xfer_len - (sil_rew_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCTSIZ)) & USB_OTG_HCTSIZ_XFRSIZ);

		hc->state = USBO_HC_XFRC;
		hc->err_count = 0;

		if((hc->ep_type == EP_TYPE_CTRL) || (hc->ep_type == EP_TYPE_BULK)){
			hc->urb_state  = URB_DONE;
//			usbo_hc_halt(husb, chnum);
			hc->chg_urb = 2;
		}
		else if(hc->ep_type == EP_TYPE_INTR){
			sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_ODDFRM);
			hc->urb_state = URB_DONE;
			hc->chg_urb = 2;
		}
		hc->toggle_in ^= 1U;
	}
	else{
		hc->state = USBO_HC_XFRC;
		hc->urb_state  = URB_DONE;
		if(hc->ep_type == EP_TYPE_BULK){
			hc->toggle_out ^= 1U; 
		}
		hc->chg_urb = 1;
	}
}

/*
 *  DMA ACK割込み処理
 */
static void
usbo_hc_ack_intr(USB_OTG_Handle_t *husb, USB_OTG_HCTypeDef *hc, int chnum, bool_t in)
{
	syslog_2(LOG_INFO, "usbo_hc_ack_intr (%d)(%d)", chnum, in);
	if(!in){
		if(hc->do_ping == 1U){
			hc->state = USBO_HC_NYET;
			usbo_hc_halt(husb, chnum);
			hc->urb_state  = URB_NOTREADY;
			if(hc->chg_urb == 0)
				hc->chg_urb++;
		}
	}
}

/*
 *  DMA NAK割込み処理
 */
static void
usbo_hc_nak_intr(USB_OTG_Handle_t *husb, USB_OTG_HCTypeDef *hc, int chnum, bool_t in)
{
	syslog_1(LOG_INFO, "usbo_hc_nak_intr (%d)", chnum);
//	if(in){
//		if(hc->ep_type == EP_TYPE_INTR){
//			usbo_hc_halt(husb, chnum);
//		}
//		else if((hc->ep_type == EP_TYPE_CTRL) || (hc->ep_type == EP_TYPE_BULK)){
			/*
			 *  チャネルの再アクティブ
			 */
//			sil_modw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHDIS, USB_OTG_HCCHAR_CHENA);
//		}
//	}
	hc->state     = USBO_HC_NAK;
	usbo_hc_halt(husb, chnum);
	hc->chg_urb = 1;
}

/*
 * DMAモード CH割込み処理
 */
static uint32_t
usbo_hc_chhltd_intr_dma(USB_OTG_Handle_t *husb, USB_OTG_HCTypeDef *hc, int chnum,
				    uint32_t shcint, bool_t in)
{
	int out_nak_enh = 0;

	/*
	 *  OUT NAK enhancement処理判定
	 */
	if (hc->speed == USB_SPEED_HIGH && !hc->ep_is_in &&
	    (hc->ep_type == EP_TYPE_CTRL || hc->ep_type == EP_TYPE_BULK)) {
		out_nak_enh = 1;
	}

	if(shcint & USB_OTG_HCINT_XFRC){
		/*
		 *  送受信割込み
		 *  ISOC/OUT/DMAの場合、ACK処理を加える
		 */
		if(hc->ep_type == EP_TYPE_ISOC && !hc->ep_is_in)
			usbo_hc_ack_intr(husb, hc, chnum, in);
		usbo_hc_xfercomp_intr(husb, hc, chnum, in);
		shcint &= ~(USB_OTG_HCINT_XFRC | USB_OTG_HCINT_ACK);
	}
	else if(shcint & USB_OTG_HCINT_STALL){	/* STALL割込み */
		hc->state     = USBO_HC_STALL;
		hc->urb_state = URB_STALL;
		usbo_hc_halt(husb, chnum);
		hc->chg_urb = 1;
		shcint &= ~USB_OTG_HCINT_STALL;
	}
	else if((shcint & USB_OTG_HCINT_TXERR)){	/* 転送エラー割込み */
		if(out_nak_enh){
			if (shcint &
			    (USB_OTG_HCINT_NYET | USB_OTG_HCINT_NAK | USB_OTG_HCINT_ACK)) {
				syslog_0(LOG_ERROR, "XactErr with NYET/NAK/ACK");
			} else {
				syslog_0(LOG_ERROR, "XactErr without NYET/NAK/ACK");
			}
		}
		else
			syslog_2(LOG_ERROR, "XactErr ch(%d) [%08x] !", chnum, shcint);
		hc->err_count++;
		hc->state = USBO_HC_XACTERR;
		usbo_hc_halt(husb, chnum);
		shcint &= ~USB_OTG_HCINT_TXERR;
	}
	else if(shcint & USB_OTG_HCINT_BBERR){	/* バブルエラー割込み */
		syslog_1(LOG_ERROR, "usbo_hc_chhltd_intr_dma BABBLE ERROR ch(%d) !", chnum);
		hc->err_count++;
		hc->state = USBO_HC_BBLERR;
		shcint &= ~USB_OTG_HCINT_BBERR;
	}
	else if(shcint & USB_OTG_HCINT_FRMOR){	/* フレーム・オーバーラン割込み */
		syslog_1(LOG_ERROR, "usbo_hc_chhltd_intr_dma FRAME OVERRUN ch(%d) !", chnum);
		hc->err_count++;
		hc->state = USBO_HC_HALTED;
		shcint &= ~USB_OTG_HCINT_FRMOR;
	}
	else if(!out_nak_enh){
		if(shcint & USB_OTG_HCINT_NYET){	/* NYET割込みハンドル */
			if(!in){
				hc->state = USBO_HC_NYET;
				hc->err_count= 0;
				hc->urb_state  = URB_NOTREADY;
				hc->do_ping = 0U;
				hc->chg_urb = 1;
			}
			usbo_hc_halt(husb, chnum);
			shcint &= ~USB_OTG_HCINT_NYET;
		}
		else if(shcint & USB_OTG_HCINT_NAK){	/* NAK割込みハンドル */
			/*
			 * If nak is not masked, it's because a non-split IN
			 * transfer is in an error state. In that case, the nak
			 * is handled by the nak interrupt handler, not here.
			 * Handle nak here for BULK/CONTROL OUT transfers, which
			 * halt on a NAK to allow rewinding the buffer pointer.
			 */
			usbo_hc_nak_intr(husb, hc, chnum, in);
			shcint &= ~USB_OTG_HCINT_NAK;
		}
		else if(shcint & USB_OTG_HCINT_ACK){	/* ACK割込みハンドル */
			/*
			 * If ack is not masked, it's because a non-split IN
			 * transfer is in an error state. In that case, the ack
			 * is handled by the ack interrupt handler, not here.
			 * Handle ack here for split transfers. Start splits
			 * halt on ACK.
			 */
			usbo_hc_ack_intr(husb, hc, chnum, in);
			shcint &= ~USB_OTG_HCINT_ACK;
		}
		else{
			if(hc->ep_type == EP_TYPE_INTR || hc->ep_type == EP_TYPE_ISOC){
				/*
				 * A periodic transfer halted with no other
				 * channel interrupts set. Assume it was halted
				 * by the core because it could not be completed
				 * in its scheduled (micro)frame.
				 */
				syslog_2(LOG_ERROR, "%s: Halt channel %d (assume incomplete periodic transfer)",
					__func__, chnum);
				usbo_hc_halt(husb, chnum);
			}
			else{
				syslog_2(LOG_ERROR, "%s: Channel %d - ChHltd set, but reason is unknown",
					__func__, chnum);
				syslog_2(LOG_ERROR, "hcint 0x%08x, intsts 0x%08x",
					shcint,
					sil_rew_mem((uint32_t *)(USBHC_BASE(husb->base, chnum)+TOFF_USBHC_HCINT)));
				goto error;
			}
		}
	}
	else{
		syslog_4(LOG_ERROR, "NYET/NAK/ACK/other in non-error case, ch(%d) hcint[%08x] pi[%08x] pm[%08x] ",
			 chnum, shcint, hc->pend_int, hc->pend_msk);
error:
		/* Failthrough: use 3-strikes rule */
		hc->err_count++;
		hc->state = USBO_HC_HALTED;
		usbo_hc_halt(husb, chnum);
	}
	if(hc->state == USBO_HC_HALTED || hc->state > USBO_HC_STALL){
		if(hc->err_count > 4){
			hc->urb_state = URB_ERROR;
			hc->err_count = 0;
		}
		else
			hc->urb_state = URB_NOTREADY;
		hc->chg_urb = 1;
	}
	return shcint;
}

/*
 *  HOST DMA用チャネル割込み要求
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 chnum チャネル番号(1-15)
 */
static void
usbo_hc_dma_irqhandler(USB_OTG_Handle_t *husb, uint8_t chnum)
{
	USB_OTG_HCTypeDef *hc = &husb->hc[chnum];
	uint32_t hcint, hcintmsk, shcint;
	bool_t in = (sil_rew_mem((uint32_t *)(USBHC_BASE(husb->base, chnum)+TOFF_USBHC_HCCHAR)) &  USB_OTG_HCCHAR_EPDIR) != 0;

	hcint = sil_rew_mem((uint32_t *)(USBHC_BASE(husb->base, chnum)+TOFF_USBHC_HCINT));
	hcintmsk = sil_rew_mem((uint32_t *)(USBHC_BASE(husb->base, chnum)+TOFF_USBHC_HCINTMSK));
	hc->pend_int |= hcint;

	shcint = hcint & (~hcintmsk);
	hcint &= hcintmsk;

	if(hcint & USB_OTG_HCINT_XFRC){
		usbo_hc_xfercomp_intr(husb, hc, chnum, in);
		/*
		 * If NYET occurred at same time as Xfer Complete, the NYET is
		 * handled by the Xfer Complete interrupt handler. Don't want
		 * to call the NYET interrupt handler in this case.
		 */
		hcint &= ~USB_OTG_HCINT_NYET;
	}

	if(hcint & USB_OTG_HCINT_CHH){
		shcint = usbo_hc_chhltd_intr_dma(husb, hc, chnum, shcint, in);
	}
	if(hcint & USB_OTG_HCINT_NAK){
		usbo_hc_nak_intr(husb, hc, chnum, in);
	}
	if(hcint & USB_OTG_HCINT_ACK){
		usbo_hc_ack_intr(husb, hc, chnum, in);
	}
	if(hcint & USB_OTG_HCINT_DTERR){
		hc->err_count++;
		hc->state = USBO_HC_DATATGLERR;
		if(hc->err_count++ > 4){
			hc->err_count = 0;
			hc->urb_state = URB_ERROR;
		}
		else
			hc->urb_state = URB_NOTREADY;
		hc->chg_urb = 1;
	}
	if((shcint & ~hcintmsk) == 0){
		hc->pend_msk |= hcintmsk;
		if(hc->err_count == 0)
			hc->pend_msk &= ~(USB_OTG_HCINTMSK_ACKM | USB_OTG_HCINTMSK_DTERRM | USB_OTG_HCINTMSK_NAKM);
		sil_andw_mem((uint32_t *)(USBHC_BASE(husb->base, chnum)+TOFF_USBHC_HCINTMSK), hcintmsk);
	}
	else
		clear_hc_int(husb->base, chnum, hc->pend_int & ~(shcint | hcintmsk));
}

/*
 *  HOSTチャネル割込み要求
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 chnum チャネル番号(1-15)
 */
static void
usbo_hc_in_irqhandler(USB_OTG_Handle_t *husb, uint8_t chnum)
{
    USB_OTG_HCTypeDef *hc = &husb->hc[chnum];
	uint32_t base = husb->base;
	uint32_t ihstatus = sil_rew_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINT));

	if((ihstatus & USB_OTG_HCINT_AHBERR) != 0){
		clear_hc_int(base, chnum, USB_OTG_HCINT_AHBERR);
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
	}
	else if((ihstatus & USB_OTG_HCINT_ACK) != 0){
		clear_hc_int(base, chnum, USB_OTG_HCINT_ACK);
	}
	else if((ihstatus & USB_OTG_HCINT_STALL) != 0){
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		hc->state = USBO_HC_STALL;
		usbo_hc_halt(husb, chnum);
		clear_hc_int(base, chnum, USB_OTG_HCINT_NAK);
		clear_hc_int(base, chnum, USB_OTG_HCINT_STALL);
	}
	else if((ihstatus & USB_OTG_HCINT_DTERR) != 0){
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		usbo_hc_halt(husb, chnum);
		clear_hc_int(base, chnum, USB_OTG_HCINT_NAK);
		hc->state = USBO_HC_DATATGLERR;
		clear_hc_int(base, chnum, USB_OTG_HCINT_DTERR);
	}

	ihstatus = sil_rew_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINT));
	if((ihstatus & USB_OTG_HCINT_FRMOR) != 0){
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		usbo_hc_halt(husb, chnum);
		clear_hc_int(base, chnum, USB_OTG_HCINT_FRMOR);
	}
	else if((ihstatus & USB_OTG_HCINT_XFRC) != 0){
		hc->state = USBO_HC_XFRC;
		hc->err_count = 0;
		clear_hc_int(base, chnum, USB_OTG_HCINT_XFRC);

		if((hc->ep_type == EP_TYPE_CTRL) || (hc->ep_type == EP_TYPE_BULK)){
			sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
			usbo_hc_halt(husb, chnum);
			clear_hc_int(base, chnum, USB_OTG_HCINT_NAK);
		}
		else if(hc->ep_type == EP_TYPE_INTR){
			sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_ODDFRM);
			hc->urb_state = URB_DONE;
			if(husb->hostchangeurbcallback != NULL)
				husb->hostchangeurbcallback(husb, chnum, husb->hc[chnum].urb_state);
		}
		hc->toggle_in ^= 1U;
	}
	else if((ihstatus & USB_OTG_HCINT_CHH) != 0){
		sil_andw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		if(hc->state == USBO_HC_XFRC){
			hc->urb_state  = URB_DONE;
		}
		else if(hc->state == USBO_HC_STALL){
			hc->urb_state  = URB_STALL;
		}
		else if((hc->state == USBO_HC_XACTERR) || (hc->state == USBO_HC_DATATGLERR)){
			if(hc->err_count++ > 3U){
				hc->err_count = 0;
				hc->urb_state = URB_ERROR;
			}
			else{
				hc->urb_state = URB_NOTREADY;
			}

			/*
			 *  チャネルの再アクティブ
			 */
			sil_modw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHDIS, USB_OTG_HCCHAR_CHENA);
		}
		clear_hc_int(base, chnum, USB_OTG_HCINT_CHH);
		hc->chg_urb = 1;
	}
	else if((ihstatus & USB_OTG_HCINT_TXERR) != 0){
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		hc->err_count++;
		hc->state = USBO_HC_XACTERR;
#if 1	/* ROI DEBUG */
		syslog_1(LOG_NOTICE, "## SET USBO_HC_XACTERR1 [%08x] ##", sil_rew_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINT)));
#endif	/* ROI DEBUG */
		usbo_hc_halt(husb, chnum);
		clear_hc_int(base, chnum, USB_OTG_HCINT_TXERR);
	}
	else if((ihstatus & USB_OTG_HCINT_NAK) != 0){
		if(hc->ep_type == EP_TYPE_INTR){
			sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
			usbo_hc_halt(husb, chnum);
		}
		else if((hc->ep_type == EP_TYPE_CTRL) || (hc->ep_type == EP_TYPE_BULK)){
			/*
			 *  チャネルの再アクティブ
			 */
			sil_modw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHDIS, USB_OTG_HCCHAR_CHENA);
		}
		hc->state = USBO_HC_NAK;
		clear_hc_int(base, chnum, USB_OTG_HCINT_NAK);
		sil_dly_nse(500*1000);
	}
}

/*
 *  HOSTチャネルOUT割込み要求
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 chnum チャネル番号(1-15)
 */
static void
usbo_hc_out_irqhandler(USB_OTG_Handle_t *husb, uint8_t chnum)
{
    USB_OTG_HCTypeDef *hc = &husb->hc[chnum];
	uint32_t base = husb->base;
	uint32_t ihstatus = sil_rew_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINT));

	if((ihstatus & USB_OTG_HCINT_AHBERR) != 0){
		clear_hc_int(base, chnum, USB_OTG_HCINT_AHBERR);
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
	}
	else if((ihstatus & USB_OTG_HCINT_ACK) != 0){
		clear_hc_int(base, chnum, USB_OTG_HCINT_ACK);
		if(hc->do_ping == 1U){
			husb->hc[chnum].state = USBO_HC_NYET;
			sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
			usbo_hc_halt(husb, chnum);
			hc->urb_state  = URB_NOTREADY;
		}
	}
	else if((ihstatus & USB_OTG_HCINT_NYET) != 0){
		hc->state = USBO_HC_NYET;
		hc->err_count= 0;
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		usbo_hc_halt(husb, chnum);
		clear_hc_int(base, chnum, USB_OTG_HCINT_NYET);
	}
	else if((ihstatus & USB_OTG_HCINT_FRMOR) != 0){
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		usbo_hc_halt(husb, chnum);
		clear_hc_int(base, chnum, USB_OTG_HCINT_FRMOR);
	}
	else if((ihstatus & USB_OTG_HCINT_XFRC) != 0){
		hc->err_count = 0;
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		usbo_hc_halt(husb, chnum);
		clear_hc_int(base, chnum, USB_OTG_HCINT_XFRC);
		hc->state = USBO_HC_XFRC;
	}
	else if((ihstatus & USB_OTG_HCINT_STALL) != 0){
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		usbo_hc_halt(husb, chnum);
		clear_hc_int(base, chnum, USB_OTG_HCINT_STALL);
		hc->state = USBO_HC_STALL;
	}
	else if((ihstatus & USB_OTG_HCINT_NAK) != 0){
		hc->err_count = 0;
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		usbo_hc_halt(husb, chnum);
		hc->state = USBO_HC_NAK;
		clear_hc_int(base, chnum, USB_OTG_HCINT_NAK);
	}
	else if((ihstatus & USB_OTG_HCINT_TXERR) != 0){
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		usbo_hc_halt(husb, chnum);
		hc->state = USBO_HC_XACTERR;
#if 1	/* ROI DEBUG */
    syslog_1(LOG_NOTICE, "## SET USBO_HC_XACTERR2 [%08x] ##", sil_rew_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINT)));
#endif	/* ROI DEBUG */
		clear_hc_int(base, chnum, USB_OTG_HCINT_TXERR);
	}
	else if((ihstatus & USB_OTG_HCINT_DTERR) != 0){
		sil_orw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
 		usbo_hc_halt(husb, chnum);
		clear_hc_int(base, chnum, USB_OTG_HCINT_NAK);
		clear_hc_int(base, chnum, USB_OTG_HCINT_DTERR);
		hc->state = USBO_HC_DATATGLERR;
	}
	else if((ihstatus & USB_OTG_HCINT_CHH) != 0){
		sil_andw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCINTMSK), USB_OTG_HCINTMSK_CHHM);
		if(hc->state == USBO_HC_XFRC){
			hc->urb_state  = URB_DONE;
			if(hc->ep_type == EP_TYPE_BULK){
				hc->toggle_out ^= 1U; 
			}
		}
		else if (husb->hc[chnum].state == USBO_HC_NAK){
			hc->urb_state  = URB_NOTREADY;
		}
		else if (hc->state == USBO_HC_NYET){
			hc->urb_state  = URB_NOTREADY;
			hc->do_ping = 0U;
		}
		else if (hc->state == USBO_HC_STALL){
			hc->urb_state  = URB_STALL;
		}
		else if((hc->state == USBO_HC_XACTERR) || (hc->state == USBO_HC_DATATGLERR)){
			if(hc->err_count++ > 3){
				hc->err_count = 0;
				hc->urb_state = URB_ERROR;
			}
			else{
				hc->urb_state = URB_NOTREADY;
			}

			/* re-activate the channel  */
			sil_modw_mem((uint32_t *)(USBHC_BASE(base, chnum)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHDIS, USB_OTG_HCCHAR_CHENA);
		}
		clear_hc_int(base, chnum, USB_OTG_HCINT_CHH);
		hc->chg_urb = 1;
	}
}

/*
 *  RX QUEUE LEVEL割込み要求
 *  parameter1 husb  USBハンドラへのポインタ
 */
static void
usbo_rxqlvl_irqhandler(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint8_t  channelnum = 0U;
	uint32_t pktsts, pktcnt;
	uint32_t temp = 0U;

	temp = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRXSTSP));
	channelnum = temp &  USB_OTG_GRXSTSP_EPNUM;  
	pktsts = (temp &  USB_OTG_GRXSTSP_PKTSTS) >> 17U;
	pktcnt = (temp &  USB_OTG_GRXSTSP_BCNT) >> 4U;

	switch(pktsts){
	case GRXSTS_PKTSTS_IN:
		/*
		 *  HOSTバッファパケット読み込み
		 */
		if((pktcnt > 0) && (husb->hc[channelnum].xfer_buff != (void  *)0U)){
			usbo_readPacket(husb->base, husb->hc[channelnum].xfer_buff, pktcnt);

			/*
			 *  マルチデータ処理
			 */
			husb->hc[channelnum].xfer_buff += pktcnt;
			husb->hc[channelnum].xfer_count  += pktcnt;

			if((sil_rew_mem((uint32_t *)(USBHC_BASE(base, channelnum)+TOFF_USBHC_HCTSIZ)) & USB_OTG_HCTSIZ_PKTCNT) > 0){
				sil_modw_mem((uint32_t *)(USBHC_BASE(base, channelnum)+TOFF_USBHC_HCCHAR), USB_OTG_HCCHAR_CHDIS, USB_OTG_HCCHAR_CHENA);
				husb->hc[channelnum].toggle_in ^= 1U;
			}
		}
		break;

	case GRXSTS_PKTSTS_DATA_TOGGLE_ERR:
	case GRXSTS_PKTSTS_IN_XFER_COMP:
	case GRXSTS_PKTSTS_CH_HALTED:
	default:
		break;
	}
}

/*
 *  HOSTポート割込み要求
 *  parameter1 husb  USBハンドラへのポインタ
 */
static void
usbo_port_irqhandler(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	volatile uint32_t hprt0, hprt0_dup;

	hprt0 = sil_rew_mem((uint32_t*)(USBPRT_BASE(base)));
	hprt0_dup = hprt0 & ~USB_OTG_HPRT_PENA;

	/*
	 *  ポートコネクトチェック
	 */
	if((hprt0 & USB_OTG_HPRT_PCDET) != 0){
		if((hprt0 & USB_OTG_HPRT_PCSTS) == USB_OTG_HPRT_PCSTS){
			sil_andw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), USB_OTG_GINTSTS_DISCINT);
			if(husb->hostconnectcallback != NULL)
				husb->hostconnectcallback(husb);
		}
	}

	/*
	 *  ポートイネーブル変化チェック
	 */
	if((hprt0 & USB_OTG_HPRT_PENCHNG) != 0){
		if((hprt0 & USB_OTG_HPRT_PENA) != 0){
			if(husb->Init.speed == USB_SPEED_FULL){
				sil_wrw_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HFIR), 60000U);
			}

			if(husb->hostconnectcallback != NULL)
				husb->hostconnectcallback(husb);

			if(husb->Init.speed == USB_SPEED_HIGH){
				sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), USB_OTG_GINTSTS_DISCINT);
			}
		}
		else{
			/*
			 *  HPRTのクリーン
			 */
			sil_andw_mem((uint32_t*)(USBPRT_BASE(base)), (USB_OTG_HPRT_PENA | USB_OTG_HPRT_PCDET |\
				USB_OTG_HPRT_PENCHNG | USB_OTG_HPRT_POCCHNG ));
			sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), USB_OTG_GINTSTS_DISCINT);
		}
	}

	/*
	 *  ポート割込みクリア
	 */
	sil_wrw_mem((uint32_t*)(USBPRT_BASE(base)), hprt0_dup);
}

/*
 *  HOST割込みハンドラ
 *  parameter1 husb  USBハンドラへのポインタ
 */
void
usbo_hcd_irqhandler(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t msk  = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK));
	uint32_t i = 0, int_status = 0;

	/*
	 *  許可しない要因の割込みは無視する
	 */
	if(get_int_status(base, msk) == 0){
		uint32_t intsts = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTSTS));
		syslog_1(LOG_EMERG, "usbo_hcd_irqhandler Error Intrrupt[%08x] !", intsts);
		return;
	}

	if((get_int_status(base, msk) & USB_OTG_GINTSTS_PXFR_INCOMPISOOUT) != 0){
		/*
		 *  未要求割込み要因クリア
		 */
		clear_host_int(base, USB_OTG_GINTSTS_PXFR_INCOMPISOOUT);
	}
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_IISOIXFR) != 0){
		/*
		 *  未要求割込み要因クリア
		 */
		clear_host_int(base, USB_OTG_GINTSTS_IISOIXFR);
	}

	if((get_int_status(base, msk) & USB_OTG_GINTSTS_PTXFE) != 0){
		/*
		 *  未要求割込み要因クリア
		 */
		clear_host_int(base, USB_OTG_GINTSTS_PTXFE);
	}
    if((get_int_status(base, msk) & USB_OTG_GINTSTS_MMIS) != 0){
		/*
		 *  未要求割込み要因クリア
		 */
		clear_host_int(base, USB_OTG_GINTSTS_MMIS);
    }
    /*
	 *  HOSTディスコネクト割込み
	 */
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_DISCINT) != 0){
#if 1	/* ROI DEBUG */
	syslog_1(LOG_NOTICE, "## usbo_hcd_irqhandler USB_OTG_GINTSTS_DISCINT[%08x] ##", get_int_status(base, msk));
#endif	/* ROI DEBUG */
		/*
		 *  HPRTのクリーンアップ
		 */
		sil_andw_mem((uint32_t*)(USBPRT_BASE(base)), (USB_OTG_HPRT_PENA | USB_OTG_HPRT_PCDET |\
			USB_OTG_HPRT_PENCHNG | USB_OTG_HPRT_POCCHNG ));
		if(husb->hostdisconnectcallback != NULL)
			husb->hostdisconnectcallback(husb);
		clear_host_int(base, USB_OTG_GINTSTS_DISCINT);
	}
	/*
	 *  HOSTポート割込み
	 */
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_HPRTINT) != 0){
		syslog_0(LOG_NOTICE, "## usbo_hcd_irqhandler USB_OTG_GINTSTS_HPRTINT ##");
		usbo_port_irqhandler(husb);
	}
	/*
	 *  Host SOF割込み
	 */
    if((get_int_status(base, msk) & USB_OTG_GINTSTS_SOF) != 0){
		USB_OTG_HCTypeDef *hc;
		for(i = 0 ; i < husb->Init.host_channels ; i++){
			hc = &husb->hc[i];
			if(hc->pend_int != 0){
				clear_hc_int(base, i, hc->pend_int);
				hc->pend_int = 0;
			}
			if(hc->pend_msk != 0){
				sil_orw_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCINTMSK), hc->pend_msk);
				hc->pend_msk = 0;
			}
			if(hc->chg_urb != 0){
				if(--hc->chg_urb == 0 && husb->hostchangeurbcallback != NULL){
					husb->hostchangeurbcallback(husb, i, hc->urb_state);
				}
			}
		}
		if(husb->hostsofcallback != NULL)
			husb->hostsofcallback(husb);
		clear_host_int(base, USB_OTG_GINTSTS_SOF);
	}
	/*
	 *  HOSTチャネル割込み
	 */
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_HCINT) != 0){
		enter_ispinlock(husb->Init.spnlockid);
		int_status = get_haint_status(base);
		if(husb->Init.dma_enable == 1){
			for(i = 0 ; i < husb->Init.host_channels; i++){
				if(int_status & (1 << i))
					usbo_hc_dma_irqhandler(husb, i);
			}
		}
		else{
			for(i = 0 ; i < husb->Init.host_channels; i++){
				if(int_status & (1 << i)){
					if((sil_rew_mem((uint32_t *)(USBHC_BASE(husb->base, i)+TOFF_USBHC_HCCHAR)) &  USB_OTG_HCCHAR_EPDIR) != 0)
						usbo_hc_in_irqhandler(husb, i);
					else
						usbo_hc_out_irqhandler(husb, i);
				}
			}
		}
		clear_host_int(base, USB_OTG_GINTSTS_HCINT);
		release_ispinlock(husb->Init.spnlockid);
	}
	/*
	 *  RXキューレベル割込み
	 */
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_RXFLVL) != 0){
		enter_ispinlock(husb->Init.spnlockid);
		sil_andw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), USB_OTG_GINTSTS_RXFLVL);
		usbo_rxqlvl_irqhandler(husb);
		sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), USB_OTG_GINTSTS_RXFLVL);
		release_ispinlock(husb->Init.spnlockid);
	}
#if 1	/* ROI DEBUG DEBUG */
    if((get_int_status(base, msk) & USB_OTG_GINTSTS_OTGINT) != 0){
		uint32_t temp = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGINT));
		syslog_1(LOG_NOTICE, "### usbo_hcd_irqhandler USB_OTG_GINTSTS_OTGINT[%08x] ##", temp);
		sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGINT), temp);
	}
#endif	/* ROI DEBUG DEBUG */
#if 1	/* ROI DEBUG */
	sil_dly_nse(1000);
#endif	/* ROI DEBUG */
}

/*******************************************************************************
                       USB DEVICE関数
*******************************************************************************/
/*
 *  USB OTG 省エネモード復帰
 */
static ER
usbo_exit_hibernation(USB_OTG_Handle_t *husb, bool_t restore)
{
	USB_OTG_GReg_Backup *gr;
	USB_OTG_HReg_Backup *hr;
	USB_OTG_DReg_Backup *dr;
	uint32_t base, pcgcctl, dctl;
	int i;

	base = husb->base;
	if (!husb->hibernation)
		return E_NOSPT;

	pcgcctl = sil_rew_mem((uint32_t *)(USBPGC_BASE(base)));
	pcgcctl &= ~USB_OTG_PCGCCTL_STOPCLK;
	sil_wrw_mem((uint32_t *)(USBPGC_BASE(base)), pcgcctl);

	pcgcctl = sil_rew_mem((uint32_t *)(USBPGC_BASE(base)));
	pcgcctl &= ~USB_OTG_PCGCTL_PWRCLMP;
	sil_wrw_mem((uint32_t *)(USBPGC_BASE(base)), pcgcctl);

	pcgcctl = sil_rew_mem((uint32_t *)(USBPGC_BASE(base)));
	pcgcctl &= ~USB_OTG_PCGCTL_RSTPDWNMODULE;
	sil_wrw_mem((uint32_t *)(USBPGC_BASE(base)), pcgcctl);

	sil_dly_nse(100*1000);
	if(!restore)
		return E_OK;

	/*
	 *  Globalレジスタリストア
	 */
	gr = &husb->global_reg_backup;
	if(!gr->valid)
		return E_OBJ;
	gr->valid = false;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGCTL), gr->gotgctl);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), gr->gintmsk);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GAHBCFG), gr->gahbcfg);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GUSBCFG), gr->gusbcfg);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRXFSIZ), gr->grxfsiz);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_DIEPTXF0_HNPTXFSIZ), gr->gnptxfsiz);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_HPTXFSIZ), gr->hptxfsiz);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GDFIFOCFG), gr->gdfifocfg);
	for (i = 0; i < MAX_EPS_CHANNELS; i++)
		sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_HPTXFSIZ+i*4), gr->dtxfsiz[i]);

	if(get_usbmode(base) == USB_OTG_MODE_HOST){	/* host mode */
		/* Restore host regs */
		hr = &husb->host_reg_backup;
		if (!hr->valid) {
			return E_OBJ;
		}
		hr->valid = false;
		sil_wrw_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HCFG), hr->hcfg);
		sil_wrw_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HAINTMSK), hr->haintmsk);

		for (i = 0; i < husb->Init.host_channels; ++i)
			sil_wrw_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCINTMSK), hr->hcintmsk[i]);

		sil_wrw_mem((uint32_t*)(USBPRT_BASE(base)), hr->hprt0);
		sil_wrw_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HFIR), hr->hfir);
	}
	else{						/* device mode */
		/* Restore dev regs */
		dr = &husb->device_reg_backup;
		if(!dr->valid) {
			return E_OBJ;
		}
		dr->valid = false;
		sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCFG), dr->dcfg);
		sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCTL), dr->dctl);
		sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINTMSK), dr->daintmsk);
		sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPMSK), dr->diepmsk);
		sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DOEPMSK), dr->doepmsk);

		for (i = 0; i < MAX_EPS_CHANNELS; i++) {
			/* Restore IN EPs */
			sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPCTL), dr->diepctl[i]);
			sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPTSIZ), dr->dieptsiz[i]);
			sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPDMA), dr->diepdma[i]);

			/* Restore OUT EPs */
			sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPCTL), dr->doepctl[i]);
			sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPTSIZ), dr->doeptsiz[i]);
			sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPDMA), dr->doepdma[i]);
		}

		/* Set the Power-On Programming done bit */
		dctl = sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCTL));
		dctl |= USB_OTG_DCTL_POPRGDNE;
		sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCTL), dctl);
	}
	return E_OK;
}

/*
 *  USB OTG 省エネモード移行
 */
static ER
usbo_enter_hibernation(USB_OTG_Handle_t *husb)
{
	USB_OTG_GReg_Backup *gr;
	USB_OTG_HReg_Backup *hr;
	USB_OTG_DReg_Backup *dr;
	uint32_t base, pcgcctl;
	int i;

	base = husb->base;
	husb->lx_state = DWC2_L2;
	if (!husb->hibernation)
		return E_NOSPT;

	/*
	 *  Globalレジスタバックアップ
	 */
	gr = &husb->global_reg_backup;
	gr->gotgctl = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGCTL));
	gr->gintmsk = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK));
	gr->gahbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GAHBCFG));
	gr->gusbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GUSBCFG));
	gr->grxfsiz = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRXFSIZ));
	gr->gnptxfsiz = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_DIEPTXF0_HNPTXFSIZ));
	gr->hptxfsiz = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_HPTXFSIZ));
	gr->gdfifocfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GDFIFOCFG));
	for (i = 0; i < MAX_EPS_CHANNELS; i++)
		gr->dtxfsiz[i] = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_HPTXFSIZ+i*4));
	gr->valid = true;

	if(get_usbmode(base) == USB_OTG_MODE_HOST){	/* host mode */
		hr = &husb->host_reg_backup;
		hr->hcfg = sil_rew_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HCFG));
		hr->haintmsk = sil_rew_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HAINTMSK));
		for (i = 0; i < husb->Init.host_channels; ++i)
			hr->hcintmsk[i] = sil_rew_mem((uint32_t *)(USBHC_BASE(base, i)+TOFF_USBHC_HCINTMSK));

		hr->hprt0 = sil_rew_mem((uint32_t*)(USBPRT_BASE(base)));
		hr->hprt0 &= ~(USB_OTG_HPRT_PENA | USB_OTG_HPRT_PCDET | USB_OTG_HPRT_PENCHNG | USB_OTG_HPRT_POCCHNG);
		hr->hfir = sil_rew_mem((uint32_t *)(USBH_BASE(base)+TOFF_USBH_HFIR));
		hr->valid = true;
	}
	else{						/* device mode */
		dr = &husb->device_reg_backup;
		dr->dcfg = sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCFG));
		dr->dctl = sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCTL));
		dr->daintmsk = sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINTMSK));
		dr->diepmsk = sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPMSK));
		dr->doepmsk = sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DOEPMSK));

		for (i = 0; i < MAX_EPS_CHANNELS; i++) {
			/* Backup IN EPs */
			dr->diepctl[i]  = sil_rew_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPCTL));
			if (dr->diepctl[i] & USB_OTG_DIEPCTL_EONUM_DPID)
				dr->diepctl[i] |= USB_OTG_DIEPCTL_SODDFRM;
			else
				dr->diepctl[i] |= USB_OTG_DIEPCTL_SD0PID_SEVNFRM;
			dr->dieptsiz[i] = sil_rew_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPTSIZ));
			dr->diepdma[i]  = sil_rew_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPDMA));

			/* Backup OUT EPs */
			dr->doepctl[i]  = sil_rew_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPCTL));
			if (dr->doepctl[i] & USB_OTG_DOEPCTL_EONUM_DPID)
				dr->doepctl[i] |= USB_OTG_DOEPCTL_SODDFRM;
			else
				dr->doepctl[i] |= USB_OTG_DOEPCTL_SD0PID_SEVNFRM;
			dr->doeptsiz[i] = sil_rew_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPTSIZ));
			dr->doepdma[i]  = sil_rew_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPDMA));
		}
		dr->valid = true;
	}

	/*
	 *  割込みペンディングクリア
	 */
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTSTS), 0xFFFFFFFF);

	/*
	 *  LOW POWER設定
	 */
	pcgcctl = sil_rew_mem((uint32_t *)(USBPGC_BASE(base)));
	pcgcctl |= USB_OTG_PCGCTL_PWRCLMP;
	sil_wrw_mem((uint32_t *)(USBPGC_BASE(base)), pcgcctl);
	sil_dly_nse(20*1000);

	pcgcctl |= USB_OTG_PCGCTL_RSTPDWNMODULE;
	sil_wrw_mem((uint32_t *)(USBPGC_BASE(base)), pcgcctl);
	sil_dly_nse(20*1000);

	pcgcctl |= USB_OTG_PCGCCTL_STOPCLK;
	sil_wrw_mem((uint32_t *)(USBPGC_BASE(base)), pcgcctl);
	return E_OK;
}

/*
 *  デバイスアクティブ変化時の差異初期化関数
 */
static void
usbo_device_init_disconnected(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t intmsk;
	uint32_t val;
	uint32_t usbcfg, gahbcfg;

	enter_spinlock(husb->Init.spnlockid);
	gahbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GAHBCFG)) & USB_OTG_GAHBCFG_GINT;
	usbo_coreinit(husb);

	/*
	 *  USBCFG初期化
	 */
	usbcfg = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GUSBCFG));
	usbcfg &= ~(USB_OTG_GUSBCFG_TOCAL | USB_OTG_GUSBCFG_PHYIF | USB_OTG_GUSBCFG_SRPCAP | USB_OTG_GUSBCFG_HNPCAP);
	/*
	 *  PHY設定
	 */
	val = (husb->phyif == GUSBCFG_PHYIF8) ? 9 : 5;
	usbcfg |= husb->phyif | (val << 10) | (7);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GUSBCFG), usbcfg);

	usbo_initFiFo(husb);
	usbo_devdisconnect(husb);

	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCFG), /*DCFG_EPMISCNT(1) | DCFG_DEVSPD_HS*/USB_SPEED_HIGH);

	/*
	 *  ペンディング割込みクリア
	 */
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGINT), 0xffffffff);
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTSTS), 0xFFFFFFFF);

	intmsk = USB_OTG_GINTMSK_SRQIM | USB_OTG_GINTMSK_GONAKEFFM |
		USB_OTG_GINTMSK_GINAKEFFM | USB_OTG_GINTMSK_USBRST |
		USB_OTG_GINTMSK_RSTDEM    | USB_OTG_GINTMSK_ENUMDNEM |
		USB_OTG_GINTMSK_OTGINT    | USB_OTG_GINTMSK_USBSUSPM |
		USB_OTG_GINTMSK_WUIM      | USB_OTG_GINTMSK_IISOIXFRM |
		USB_OTG_GINTMSK_PXFRM_IISOOXFRM | USB_OTG_GINTMSK_CIDSCHGM;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), intmsk);

	if(husb->Init.dma_enable != 0)
		gahbcfg |= USB_OTG_GAHBCFG_DMAEN | GAHBCFG_HBSTLEN_INCR4;
	else if(husb->dedicated_fifos)
		gahbcfg |= USB_OTG_GAHBCFG_TXFELVL | USB_OTG_GAHBCFG_PTXFELVL;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GAHBCFG), gahbcfg);

	/*
	 *  DIEPMSK初期設定
	 */
	val =	USB_OTG_DIEPMSK_EPDM | USB_OTG_DIEPMSK_XFRCM |
			USB_OTG_DIEPMSK_TOM | USB_OTG_DIEPMSK_AHBERRM;
	if(husb->dedicated_fifos && husb->Init.dma_enable == 0)
		val |= USB_OTG_DIEPMSK_ITTXFEMSK;
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPMSK), val);

	/*
	 *  DOEPMSK初期設定
	 */
	val = USB_OTG_DOEPMSK_EPDM | USB_OTG_DOEPMSK_AHBERRM | USB_OTG_DOEPMSK_STUPM;
	if(husb->Init.dma_enable != 0)
		val |= USB_OTG_DIEPMSK_XFRCM;
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DOEPMSK), val);
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINTMSK), 0);

	/* enable in and out endpoint interrupts */
	intmsk |= USB_OTG_GINTMSK_OEPINT | USB_OTG_GINTMSK_IEPINT;

	/*
	 *  スレーブモード時、RXFIFO有効化
	 */
	if(husb->Init.dma_enable == 0)
		intmsk |= USB_OTG_GINTMSK_RXFLVLM;

	/*
	 *  SOF割込み許可設定
	 */
	if(husb->Init.sof_enable)
		intmsk |= USB_OTG_GINTMSK_SOFM;
	sil_wrw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), intmsk);
	syslog_2(LOG_NOTICE, "## GINTMSK[%08x] dma(%d) ##", intmsk, husb->Init.dma_enable);

	sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCTL), USB_OTG_DCTL_POPRGDNE);
	sil_dly_nse(10*1000);  /* see openiboot */
	sil_andw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCTL), USB_OTG_DCTL_POPRGDNE);

	/* clear global NAKs */
	val = USB_OTG_DCTL_CGONAK | USB_OTG_DCTL_CGINAK;
	sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCTL), val);

#if 1	/* ROI DEBUG DEBUG */
	if(husb->Init.dma_enable == 1){
		/*
		 *  DMAモードならスレッシュホールドパラメータを設定
		 */
		sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DTHRCTL), (USB_OTG_DTHRCTL_TXTHRLEN_6 | USB_OTG_DTHRCTL_RXTHRLEN_6));
		sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DTHRCTL), (USB_OTG_DTHRCTL_RXTHREN | USB_OTG_DTHRCTL_ISOTHREN | USB_OTG_DTHRCTL_NONISOTHREN));
		val = sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DTHRCTL));
	}
#endif	/* ROI DEBUG DEBUG */
	husb->lx_state = DWC2_L0;
	release_spinlock(husb->Init.spnlockid);

	/* must be at-least 3ms to allow bus to see disconnect */
	sil_dly_nse(1000*1000);
}

/*
 *  USBデバイス アクティブ、インアクティブ設定
 */
static void
usbo_pcd_control(USB_OTG_Handle_t *husb, int is_active)
{
	enter_spinlock(husb->Init.spnlockid);
	/*
	 * If controller is hibernated, it must exit from hibernation
	 * before being initialized / de-initialized
	 */
	if (husb->lx_state == DWC2_L2)
		usbo_exit_hibernation(husb, false);

	if(is_active){
		if(husb->Init.usb_otg_mode == USB_OTG_MODE_DEVICE)
			husb->op_state = OTG_STATE_B_PERIPHERAL;
		else
			husb->op_state = OTG_STATE_A_PERIPHERAL;
		release_spinlock(husb->Init.spnlockid);

		usbo_device_init_disconnected(husb);
		if(husb->enabled)
			usbo_devconnect(husb);
	}
	else{
		if(husb->connected){
			husb->connected = 0;
			husb->lx_state = DWC2_L3;
		}
		release_spinlock(husb->Init.spnlockid);
	}
}


/*
 *  デバイスコネクト
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_devconnect(USB_OTG_Handle_t *husb)
{
	sil_andw_mem((uint32_t *)(USBD_BASE(husb->base)+TOFF_USBD_DCTL), USB_OTG_DCTL_SDIS);
	return E_OK;
}

/*
 *  デバイスディスコネクト
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_devdisconnect(USB_OTG_Handle_t *husb)
{
	sil_orw_mem((uint32_t *)(USBD_BASE(husb->base)+TOFF_USBD_DCTL), USB_OTG_DCTL_SDIS);
	return E_OK; 
}

/*
 *  アクティブエンドポイント設定
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 ep    エンドポイント構造体へのポインタ
 *  return     常にE_OK
 */
ER
usbo_activateEndpoint(USB_OTG_Handle_t *husb, USB_OTG_EPTypeDef *ep)
{
	uint32_t base = husb->base;

	if(ep->is_in == 1){
		sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINTMSK), USB_OTG_DAINTMSK_IEPM & (1 << (ep->num)));
		if(((sil_rew_mem((uint32_t *)(USBIEP_BASE(base, ep->num)+TOFF_USBEI_DIEPCTL))) & USB_OTG_DIEPCTL_USBAEP) == 0){
			sil_orw_mem((uint32_t *)(USBIEP_BASE(base, ep->num)+TOFF_USBEI_DIEPCTL), ((ep->maxpacket & USB_OTG_DIEPCTL_MPSIZ ) | (ep->type << 18U) |\
				((ep->num) << 22U) | (USB_OTG_DIEPCTL_SD0PID_SEVNFRM) | (USB_OTG_DIEPCTL_USBAEP))); 
		}
	}
	else{
		sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINTMSK), USB_OTG_DAINTMSK_OEPM & ((1 << (ep->num)) << 16));
		if(((sil_rew_mem((uint32_t *)(USBOEP_BASE(base, ep->num)+TOFF_USBEO_DOEPCTL))) & USB_OTG_DIEPCTL_USBAEP) == 0){
			sil_orw_mem((uint32_t *)(USBOEP_BASE(base, ep->num)+TOFF_USBEO_DOEPCTL), ((ep->maxpacket & USB_OTG_DOEPCTL_MPSIZ ) | (ep->type << 18U) |\
				(USB_OTG_DIEPCTL_SD0PID_SEVNFRM)| (USB_OTG_DOEPCTL_USBAEP)));
		}
	}
	return E_OK;
}

/*
 *  ディスアクティブエンドポイント設定
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 ep    エンドポイント構造体へのポインタ
 *  return     常にE_OK
 */
ER
usbo_deactivateEndpoint(USB_OTG_Handle_t *husb, USB_OTG_EPTypeDef *ep)
{
	uint32_t base = husb->base;

	if(ep->is_in == 1){
		sil_andw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINTMSK), (USB_OTG_DAINTMSK_IEPM & (1 << (ep->num))));
		sil_andw_mem((uint32_t *)(USBIEP_BASE(base, ep->num)+TOFF_USBEI_DIEPCTL), USB_OTG_DIEPCTL_USBAEP);
	}
	else{
		sil_andw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINTMSK), (USB_OTG_DAINTMSK_OEPM & ((1 << (ep->num)) << 16)));
		sil_andw_mem((uint32_t *)(USBOEP_BASE(base, ep->num)+TOFF_USBEO_DOEPCTL), USB_OTG_DOEPCTL_USBAEP);
	}
	return E_OK;
}

/*
 *  エンドポイントにステール状態を設定
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 ep    エンドポイント構造体へのポインタ
 *  return     常にE_OK
 */
ER
usbo_epsetStall(USB_OTG_Handle_t *husb, USB_OTG_EPTypeDef *ep)
{
	uint32_t base = husb->base;

	if(ep->is_in == 1){
		if((sil_rew_mem((uint32_t *)(USBIEP_BASE(base, ep->num)+TOFF_USBEI_DIEPCTL)) & USB_OTG_DIEPCTL_EPENA) == 0){
			sil_andw_mem((uint32_t *)(USBIEP_BASE(base, ep->num)+TOFF_USBEI_DIEPCTL), USB_OTG_DIEPCTL_EPDIS);
		}
		sil_orw_mem((uint32_t *)(USBIEP_BASE(base, ep->num)+TOFF_USBEI_DIEPCTL), USB_OTG_DIEPCTL_STALL);
	}
	else{
		if((sil_rew_mem((uint32_t *)(USBOEP_BASE(base, ep->num)+TOFF_USBEO_DOEPCTL)) & USB_OTG_DOEPCTL_EPENA) == 0){
			sil_andw_mem((uint32_t *)(USBOEP_BASE(base, ep->num)+TOFF_USBEO_DOEPCTL), USB_OTG_DOEPCTL_EPDIS);
		}
		sil_orw_mem((uint32_t *)(USBOEP_BASE(base, ep->num)+TOFF_USBEO_DOEPCTL), USB_OTG_DOEPCTL_STALL);
	}
	return E_OK;
}

/*
 *  エンドポイントのステール状態をクリア
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 ep    エンドポイント構造体へのポインタ
 *  return     常にE_OK
 *
 */
ER
usbo_epclearStall(USB_OTG_Handle_t *husb, USB_OTG_EPTypeDef *ep)
{
	uint32_t base = husb->base;

	if(ep->is_in == 1){
		sil_andw_mem((uint32_t *)(USBIEP_BASE(base, ep->num)+TOFF_USBEI_DIEPCTL), USB_OTG_DIEPCTL_STALL);
		if(ep->type == EP_TYPE_INTR || ep->type == EP_TYPE_BULK){
			sil_orw_mem((uint32_t *)(USBIEP_BASE(base, ep->num)+TOFF_USBEI_DIEPCTL), USB_OTG_DIEPCTL_SD0PID_SEVNFRM);	/* DATA0 */
		}
	}
	else{
		sil_andw_mem((uint32_t *)(USBOEP_BASE(base, ep->num)+TOFF_USBEO_DOEPCTL), USB_OTG_DOEPCTL_STALL);
		if(ep->type == EP_TYPE_INTR || ep->type == EP_TYPE_BULK){
			sil_orw_mem((uint32_t *)(USBOEP_BASE(base, ep->num)+TOFF_USBEO_DOEPCTL), USB_OTG_DOEPCTL_SD0PID_SEVNFRM);	/* DATA0 */
		}
	}
	return E_OK;
}

/**
 *  USBにデバイスアドレスを設定
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 addree デバイスアドレス(0-255)
 *  return     常にE_OK
 */
ER
usbo_setDevAddress(USB_OTG_Handle_t *husb, uint8_t address)
{
	husb->connected = 1;
	sil_andw_mem((uint32_t *)(USBD_BASE(husb->base)+TOFF_USBD_DCFG), USB_OTG_DCFG_DAD);
	sil_orw_mem((uint32_t *)(USBD_BASE(husb->base)+TOFF_USBD_DCFG), ((address << 4) & USB_OTG_DCFG_DAD));
	return E_OK;
}

/*
 *  USBデバイススピードを取り出す
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     speed デバイススピード USB_SPEED_HIGH: High speed mode
 *                                    USB_SPEED_FULL: Full speed mode
 *                                    USB_SPEED_LOW: Low speed mode
 */
uint8_t
usbo_getDevSpeed(USB_OTG_Handle_t *husb)
{
	uint8_t  speed = 0;
	uint32_t status = sil_rew_mem((uint32_t *)(USBD_BASE(husb->base)+TOFF_USBD_DSTS));

	if((status & USB_OTG_DSTS_ENUMSPD) == DSTS_ENUMSPD_HS_PHY_30MHZ_OR_60MHZ){
		speed = USB_SPEED_HIGH;
	}
	else if(((status & USB_OTG_DSTS_ENUMSPD) == DSTS_ENUMSPD_FS_PHY_30MHZ_OR_60MHZ) || ((status & USB_OTG_DSTS_ENUMSPD) == DSTS_ENUMSPD_FS_PHY_48MHZ)){
		speed = USB_SPEED_FULL;
	}
	else if((status & USB_OTG_DSTS_ENUMSPD) == DSTS_ENUMSPD_LS_PHY_6MHZ){
		speed = USB_SPEED_LOW;
	}
	return speed;
}

/*
 *  EP0-OUTスタート
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 psetup セットアップパケットデータへのポインタ
 *  return     常にE_OK
 */
ER
usbo_ep0_outstart(USB_OTG_Handle_t *husb, uint8_t *psetup)
{
	uint32_t base = husb->base;

	sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, 0)+TOFF_USBEO_DOEPTSIZ), 0);
	sil_orw_mem((uint32_t *)(USBOEP_BASE(base, 0)+TOFF_USBEO_DOEPTSIZ), (USB_OTG_DOEPTSIZ_PKTCNT & (1U << 19U)));
	sil_orw_mem((uint32_t *)(USBOEP_BASE(base, 0)+TOFF_USBEO_DOEPTSIZ), (3U * 8U));
	sil_orw_mem((uint32_t *)(USBOEP_BASE(base, 0)+TOFF_USBEO_DOEPTSIZ), USB_OTG_DOEPTSIZ_STUPCNT);

	if(husb->Init.dma_enable == 1){
		flushinvalidatedcache_by_addr(psetup, 64);
		sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, 0)+TOFF_USBEO_DOEPDMA), (uint32_t)psetup);
 		/*
		 *  EP0有効化
		 */
		sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, 0)+TOFF_USBEO_DOEPCTL), 0x80008000);
	}
	return E_OK;
}

/*
 *  EP0転送スタート
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 ep    エンドポイント構造体へのポインタ
 *  return     常にE_OK
 */
ER
usbo_ep0startxfer(USB_OTG_Handle_t *husb, USB_OTG_EPTypeDef *ep)
{
	uint32_t base   = husb->base;
	uint8_t  epnum  = ep->num;

	if(ep->is_in == 1){	/* INエンドポイント */
		if(ep->xfer_len == 0){	/* ZERO-LENGTHパケット */
			sil_andw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), USB_OTG_DIEPTSIZ_PKTCNT);
			sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), (USB_OTG_DIEPTSIZ_PKTCNT & (1 << 19)));
			sil_andw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), USB_OTG_DIEPTSIZ_XFRSIZ);
		}
		else{	/* 通常パケット */
			/*
			 *  パケット数とデータ長に分けて設定(EP0なのでパケット数は1)
			 */
			sil_andw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), USB_OTG_DIEPTSIZ_XFRSIZ);
			sil_andw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), USB_OTG_DIEPTSIZ_PKTCNT);
			if(ep->xfer_len > ep->maxpacket){
				ep->xfer_len = ep->maxpacket;
			}
			sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), (USB_OTG_DIEPTSIZ_PKTCNT & (1 << 19)));
			sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), (USB_OTG_DIEPTSIZ_XFRSIZ & ep->xfer_len));
		}

		if(husb->Init.dma_enable == 1){
			flushinvalidatedcache_by_addr(((uint8_t *)ep->dma_addr), ep->xfer_len);
			sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPDMA), ep->dma_addr);
		}
		else{
			/*
			 *  DMAモードでない場合は、EMPTY割込みを設定
			 */
			if(ep->xfer_len > 0){
				sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPEMPMSK), (1 << (ep->num)));
			}
		}
		/*
		 *  クリアNAK、エンドポイントイネーブル
		 */
		sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPCTL), (USB_OTG_DIEPCTL_CNAK | USB_OTG_DIEPCTL_EPENA));
	}
	else{	 /* OUTエンドポイント */
		/*
		 *  パケット数とデータ長に分けて設定(EP0なのでパケット数は1)
		 */
		sil_andw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ), USB_OTG_DOEPTSIZ_XFRSIZ);
		sil_andw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ), USB_OTG_DOEPTSIZ_PKTCNT);

		if(ep->xfer_len > 0U){
			ep->xfer_len = ep->maxpacket;
		}

		sil_orw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ), (USB_OTG_DOEPTSIZ_PKTCNT & (1 << 19)));
		sil_orw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ), (USB_OTG_DOEPTSIZ_XFRSIZ & (ep->maxpacket)));

		if(husb->Init.dma_enable == 1){
			flushinvalidatedcache_by_addr(ep->xfer_buff, ep->xfer_len);
			sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPDMA), (uint32_t)(ep->xfer_buff));
		}
		/*
		 *  クリアNAK、エンドポイントイネーブル
		 */
		sil_orw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPCTL), (USB_OTG_DOEPCTL_CNAK | USB_OTG_DOEPCTL_EPENA));
	}
	return E_OK;
}

/*
 *  EPn転送スタート
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 ep    エンドポイント構造体へのポインタ
 *  return     常にE_OK
 */
ER
usbo_epstartxfer(USB_OTG_Handle_t *husb, USB_OTG_EPTypeDef *ep)
{
	uint32_t base   = husb->base;
	uint16_t pktcnt = 0;
	uint8_t  epnum  = ep->num;

	if(ep->is_in == 1){	/* INエンドポイント */
		if(ep->xfer_len == 0){	/* ZERO-LENGTHパケット */
			sil_andw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), USB_OTG_DIEPTSIZ_PKTCNT);
			sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), (USB_OTG_DIEPTSIZ_PKTCNT & (1 << 19)));
			sil_andw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), USB_OTG_DIEPTSIZ_XFRSIZ);
		}
		else{	/* 通常パケット */
			/*
			 *  パケット数とデータ長に分けて設定
			 */
			sil_andw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), USB_OTG_DIEPTSIZ_XFRSIZ);
			sil_andw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), USB_OTG_DIEPTSIZ_PKTCNT);
			sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), (USB_OTG_DIEPTSIZ_PKTCNT & (((ep->xfer_len + ep->maxpacket -1)/ ep->maxpacket) << 19)));
			sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), (USB_OTG_DIEPTSIZ_XFRSIZ & ep->xfer_len));

			if(ep->type == EP_TYPE_ISOC){
				sil_andw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), USB_OTG_DIEPTSIZ_MULCNT);
				sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPTSIZ), (USB_OTG_DIEPTSIZ_MULCNT & (1 << 29)));
			}
		}

		if(husb->Init.dma_enable == 1){
			flushinvalidatedcache_by_addr(((uint8_t *)ep->dma_addr), ep->xfer_len);
			sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPDMA), ep->dma_addr);
		}
		else{
			/*
			 *  EPがISOCでない場合、送信データありならばEMPTY割込みを設定
			 */
			if(ep->type != EP_TYPE_ISOC && ep->xfer_len > 0){
				sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPEMPMSK), (1 << (epnum)));
			}
		}

		if(ep->type == EP_TYPE_ISOC){
			if((sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DSTS)) & (1 << 8)) == 0){
				sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPCTL), USB_OTG_DIEPCTL_SODDFRM);
			}
			else{
				sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPCTL), USB_OTG_DIEPCTL_SD0PID_SEVNFRM);
			}
		}
		/*
		 *  クリアNAK、エンドポイントイネーブル
		 */
		sil_orw_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPCTL), (USB_OTG_DIEPCTL_CNAK | USB_OTG_DIEPCTL_EPENA));

		if(ep->type == EP_TYPE_ISOC){
			usbo_writePacket(base, ep->xfer_buff, epnum, ep->xfer_len, husb->Init.dma_enable);
		}
	}
	else{	 /* OUTエンドポイント */
		/*
		 *  パケット数とデータ長に分けて設定
		 */
		sil_andw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ), USB_OTG_DOEPTSIZ_XFRSIZ);
		sil_andw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ), USB_OTG_DOEPTSIZ_PKTCNT);

		if(ep->xfer_len == 0){	/* ZERO-LENGTHパケット */
			sil_orw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ), (USB_OTG_DOEPTSIZ_XFRSIZ & ep->maxpacket));
			sil_orw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ), (USB_OTG_DOEPTSIZ_PKTCNT & (1 << 19)));
		}
		else{	/* 通常パケット */
			pktcnt = (ep->xfer_len + ep->maxpacket - 1)/ ep->maxpacket; 
			sil_orw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ), (USB_OTG_DOEPTSIZ_PKTCNT & (pktcnt << 19)));
			sil_orw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ), (USB_OTG_DOEPTSIZ_XFRSIZ & (ep->maxpacket * pktcnt)));
		}

		if(husb->Init.dma_enable == 1){
			flushinvalidatedcache_by_addr(ep->xfer_buff, ep->xfer_len);
			sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPDMA), (uint32_t)(ep->xfer_buff));
		}

		if(ep->type == EP_TYPE_ISOC){
			if((sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DSTS)) & (1 << 8)) == 0){
				sil_orw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPCTL), USB_OTG_DOEPCTL_SODDFRM);
			}
			else{
				sil_orw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPCTL), USB_OTG_DOEPCTL_SD0PID_SEVNFRM);
			}
		}
		/*
		 *  クリアNAK、エンドポイントイネーブル
		 */
		sil_orw_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPCTL), (USB_OTG_DOEPCTL_CNAK | USB_OTG_DOEPCTL_EPENA));
	}
	return E_OK;
}

/*
 *  USBデバイス初期化
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_devinit(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t i = 0U;
	USB_OTG_HwParam_t *hw = &husb->hw_params;

	enter_spinlock(husb->Init.spnlockid);
	/* Check against largest possible value. */
	if (husb->pcd_np_g_tx_fifo_sz > hw->dev_nperio_tx_fifo_size) {
		husb->pcd_np_g_tx_fifo_sz = hw->dev_nperio_tx_fifo_size;
	}

	/* Dump fifo information */
	syslog_1(LOG_NOTICE, "NonPeriodic TXFIFO size: %d", husb->pcd_np_g_tx_fifo_sz);
	syslog_1(LOG_NOTICE, "RXFIFO size: %d", husb->pcd_rx_fifo_sz);

	if(husb->Init.usb_otg_mode == USB_OTG_MODE_DEVICE)
		husb->op_state = OTG_STATE_B_PERIPHERAL;

	husb->fifo_mem = hw->total_fifo_size;
	husb->dedicated_fifos = hw->en_multiple_tx_fifo;

	/*
	 *  デバイスに関係する、すべての割込みをクリアする
	 */
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPMSK),
			(USB_OTG_DIEPMSK_TOM | USB_OTG_DIEPMSK_AHBERRM | USB_OTG_DIEPMSK_EPDM | USB_OTG_DIEPMSK_XFRCM));
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DOEPMSK),
			(USB_OTG_DOEPMSK_STUPM | USB_OTG_DOEPMSK_AHBERRM | USB_OTG_DOEPMSK_EPDM | USB_OTG_DOEPMSK_XFRCM));
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINT), 0xFFFFFFFF);
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINTMSK), 0);

	for(i = 0 ; i < husb->Init.dev_endpoints ; i++){
		if((sil_rew_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPCTL)) & USB_OTG_DIEPCTL_EPENA) != 0){
			sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPCTL), (USB_OTG_DIEPCTL_EPDIS | USB_OTG_DIEPCTL_SNAK));
		}
		else{
			sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPCTL), 0);
		}
		sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPTSIZ), 0);
		sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPINT), 0xFF);
	}

	for(i = 0U; i < husb->Init.dev_endpoints; i++){
		if((sil_rew_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPCTL)) & USB_OTG_DOEPCTL_EPENA) != 0){
			sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPCTL), (USB_OTG_DOEPCTL_EPDIS | USB_OTG_DOEPCTL_SNAK));
		}
		else{
			sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPCTL), 0);
		}
		sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPTSIZ), 0);
		sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPINT), 0xFF);
	}

	husb->enabled = 1;
	release_spinlock(husb->Init.spnlockid);
	usbo_device_init_disconnected(husb);
	dly_tsk(25);
	return E_OK;
}

/*
 *  USBデバイス停止
 *  parameter1 husb  USBハンドラへのポインタ
 *  return     常にE_OK
 */
ER
usbo_stopdevice(USB_OTG_Handle_t *husb)
{
	uint32_t base = husb->base;
	uint32_t i;

	enter_spinlock(husb->Init.spnlockid);
	/*
	 *  エンドポイントの割込みクリア
	 */
	for(i = 0 ; i < 15 ; i++){
		sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPINT), 0xFF);
		sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, i)+TOFF_USBEO_DOEPINT), 0xFF);
	}
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINT), 0xFFFFFFFF);

	/*
	 *  エンドポイントの割込み設定
	 */
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPMSK), 0);
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DOEPMSK), 0);
	sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINTMSK), 0);

	husb->enabled = 0;
	release_spinlock(husb->Init.spnlockid);
	usbo_pcd_control(husb, 0);
	return E_OK;
}

/*
 *  LPM初期設定
 */
ER
usbo_init_lpm(USB_OTG_Handle_t *husb)
{
	return E_OK;
}

/*
 *  USB EP-IN割込み要因取得
 *  parameter1 base USB OTGレジスタベースアドレス
 *  parameter2 epnum エンドポイント番号
 *  return     EP-IN割込み要因
 */
static uint32_t
usbo_devInEPInterrupt(uint32_t base , uint8_t epnum)
{
	uint32_t msk, emp;

	msk = sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPMSK));
	emp = sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPEMPMSK));
	msk |= ((emp >> epnum) & 0x1) << 7;
	return sil_rew_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DIEPINT)) & msk;
}

/*
 *  USB EP-OUT割込み要因取得
 *  parameter1 base USB OTGレジスタベースアドレス
 *  parameter2 epnum エンドポイント番号
 *  return     EP-OUT割込み要因
 */
static uint32_t
usbo_devOutEPInterrupt(uint32_t base, uint8_t epnum)
{
	uint32_t v;
	v = sil_rew_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPINT));
	return v & sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DOEPMSK));
}


/*
 *  次のパケットを送信FIFOに設定
 *  parameter1 husb  USBハンドラへのポインタ
 *  parameter2 epnum エンドポイント番号
 *  return     常にE_OK
 */
static ER
usbo_writeemptyTxfifo(USB_OTG_Handle_t *husb, uint32_t epnum)
{
	uint32_t  base = husb->base;
	USB_OTG_EPTypeDef *ep;
	int32_t len = 0U;
	uint32_t len32b;

	ep = &husb->IN_ep[epnum];
	len = ep->xfer_len - ep->xfer_count;

	if(len > ep->maxpacket){
		len = ep->maxpacket;
	}
	len32b = (len + 3) / 4;

	while((sil_rew_mem((uint32_t *)(USBIEP_BASE(base, epnum)+TOFF_USBEI_DTXFSTS)) & USB_OTG_DTXFSTS_INEPTFSAV) > len32b
          && ep->xfer_count < ep->xfer_len && ep->xfer_len != 0){
		/*
		 * 送信FIFOへのデータ書き込み
		 */
		len = ep->xfer_len - ep->xfer_count;
		if(len > ep->maxpacket){
			len = ep->maxpacket;
		}
		len32b = (len + 3) / 4;
		usbo_writePacket(base, ep->xfer_buff, epnum, len, husb->Init.dma_enable); 
		ep->xfer_buff  += len;
		ep->xfer_count += len;
	}

	if(len <= 0){	/* 最後のパケットならばEMPTY割込みをマスク */
		sil_andw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPEMPMSK), (1 << epnum));
	}
	return E_OK;
}

/*
 *  DEVICE割込みハンドラ
 *  parameter1 husb  USBハンドラへのポインタ
 */
void
usbo_pcd_irqhandler(USB_OTG_Handle_t *husb)
{
	USB_OTG_EPTypeDef *ep;
	uint32_t base = husb->base;
	uint32_t msk  = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK));
	uint32_t i = 0U, ep_intr = 0U, epint = 0U, epnum = 0U;
	uint32_t status, temp = 0;

	/*
	 *  割込みエラーチェック
	 */
	if((get_int_status(base, msk) == 0)){
		uint32_t intsts = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTSTS));
		syslog_1(LOG_EMERG, "usbo_pcd_irqhandler Error Intrrupt[%08x] !", intsts);
		return;
	}

    if((get_int_status(base, msk) & USB_OTG_GINTSTS_MMIS) != 0){
		/*
		 *  エラー割込み要因クリア
		 */
		clear_dev_int(base, USB_OTG_GINTSTS_MMIS);
	}
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_OEPINT) != 0){
		/*
		 *  EP-OUT割込み
		 */
		ep_intr = get_outep_int_status(base);
		for(epnum = 0 ; epnum < 16 ; epnum++){
			if((ep_intr & (1<<epnum)) != 0){
				epint = usbo_devOutEPInterrupt(base, epnum);
				if((epint & USB_OTG_DOEPINT_XFRC) == USB_OTG_DOEPINT_XFRC){
					clear_outep_int(base, epnum, USB_OTG_DOEPINT_XFRC);
					if(husb->Init.dma_enable == 1U){
						husb->OUT_ep[epnum].xfer_count = husb->OUT_ep[epnum].maxpacket- (sil_rew_mem((uint32_t *)(USBOEP_BASE(base, epnum)+TOFF_USBEO_DOEPTSIZ)) & USB_OTG_DOEPTSIZ_XFRSIZ); 
						husb->OUT_ep[epnum].xfer_buff += husb->OUT_ep[epnum].maxpacket;
					}
					if(husb->devdataoutstagecallback != NULL)
						husb->devdataoutstagecallback(husb, epnum);
					if(husb->Init.dma_enable == 1 && epnum == 0 && husb->OUT_ep[epnum].xfer_len == 0){
						/*
						 *  DMAモードEP0 ZERO LENGTHパケット送信
						 */
						usbo_ep0_outstart(husb, (uint8_t *)husb->Setup);
					}
				}
				if((epint & USB_OTG_DOEPINT_STUP) != 0){
					/*
					 *  SETUPパケット通知
					 */
					if(husb->devsetupstagecallback != NULL)
						husb->devsetupstagecallback(husb);
					clear_outep_int(base, epnum, USB_OTG_DOEPINT_STUP);
				}
				if((epint & USB_OTG_DOEPINT_OTEPDIS) != 0){
					clear_outep_int(base, epnum, USB_OTG_DOEPINT_OTEPDIS);
				}
			}
		}
	}
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_IEPINT) != 0){
		/*
		 *  EP-IN割込み
		 */
		ep_intr = get_inep_int_status(base);
		for(epnum = 0 ; epnum < 16 ; epnum++){
			if((ep_intr & (1<<epnum)) != 0){ /* In ITR */
				epint = usbo_devInEPInterrupt(base, epnum);
				if((epint & USB_OTG_DIEPINT_XFRC) != 0){
					sil_andw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPEMPMSK), (1<<epnum));
					clear_inep_int(base, epnum, USB_OTG_DIEPINT_XFRC);
					if(husb->Init.dma_enable == 1){
						husb->IN_ep[epnum].xfer_buff += husb->IN_ep[epnum].maxpacket; 
					}
					if(husb->devdatainstagecallback != NULL)
						husb->devdatainstagecallback(husb, epnum);
					if(husb->Init.dma_enable == 1 && epnum == 0 && husb->IN_ep[epnum].xfer_len == 0){
						/*
						 *  DMAモードEP0 ZERO LENGTHパケット送信
						 */
						usbo_ep0_outstart(husb, (uint8_t *)husb->Setup);
					}
				}
				if((epint & USB_OTG_DIEPINT_TOC) != 0){
					clear_inep_int(base, epnum, USB_OTG_DIEPINT_TOC);
				}
				if((epint & USB_OTG_DIEPINT_ITTXFE) != 0){
					clear_inep_int(base, epnum, USB_OTG_DIEPINT_ITTXFE);
				}
				if((epint & USB_OTG_DIEPINT_INEPNE) != 0){
					clear_inep_int(base, epnum, USB_OTG_DIEPINT_INEPNE);
				}
				if((epint & USB_OTG_DIEPINT_EPDISD) != 0){
					clear_inep_int(base, epnum, USB_OTG_DIEPINT_EPDISD);
				}
				if((epint & USB_OTG_DIEPINT_TXFE) != 0){
					usbo_writeemptyTxfifo(husb , epnum);
				}
			}
		}
	}
	/*
	 *  RESUME割込み
	 */
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_WKUINT) != 0){
		usbo_exit_hibernation(husb, true);		/* 省エネ復帰 */
		if(husb->devresumecallback != NULL)
			husb->devresumecallback(husb);
		clear_dev_int(base, USB_OTG_GINTSTS_WKUINT);
	}
    /*
	 *  SUSPEND割込み
	 */
    if((get_int_status(base, msk) & USB_OTG_GINTSTS_USBSUSP) != 0){
		if((sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DSTS)) & USB_OTG_DSTS_SUSPSTS) != 0){
			usbo_enter_hibernation(husb);		/* 省エネ移行 */
			if(husb->devsuspendcallback != NULL && husb->connected)
				husb->devsuspendcallback(husb);
		}
		clear_dev_int(base, USB_OTG_GINTSTS_USBSUSP);
	}
	/*
	 * USB-RESET割込み
	 */
	status = get_int_status(base, msk);
	if((status & USB_OTG_GINTSTS_RSTDET) != 0){
		clear_dev_int(base, USB_OTG_GINTSTS_RSTDET);
		if(husb->lx_state == DWC2_L2) {
			usbo_exit_hibernation(husb, true);
			husb->lx_state = DWC2_L0;
		}
	}
	if((status & (USB_OTG_GINTSTS_USBRST|USB_OTG_GINTSTS_RSTDET)) != 0){
		sil_andw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCTL), USB_OTG_DCTL_RWUSIG);
		usbo_flushTxFifo(husb, 0U);
		for(i = 0; i < husb->Init.dev_endpoints ; i++){
			sil_wrw_mem((uint32_t *)(USBIEP_BASE(base, i)+TOFF_USBEI_DIEPINT), 0xFF);
			sil_wrw_mem((uint32_t *)(USBOEP_BASE(base, 0)+TOFF_USBEO_DOEPINT), 0xFF);
		}
		sil_wrw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINT), 0xFFFFFFFF);
		sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DAINTMSK), 0x10001);
		sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DOEPMSK), (USB_OTG_DOEPMSK_STUPM | USB_OTG_DOEPMSK_XFRCM | USB_OTG_DOEPMSK_EPDM));
		sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DIEPMSK), (USB_OTG_DIEPMSK_TOM | USB_OTG_DIEPMSK_XFRCM | USB_OTG_DIEPMSK_EPDM));
		/*
		 *  デフォルトアドレスをゼロに設定
		 */
		sil_andw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCFG), USB_OTG_DCFG_DAD);
		/*
		 *  SETUPパケット受信設定
		 */
		usbo_ep0_outstart(husb, (uint8_t *)husb->Setup);
		clear_dev_int(base, USB_OTG_GINTSTS_USBRST);
	}
	/*
	 *  ENUMERATIONハンドル終了割込み
	 */
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_ENUMDNE) != 0){
		/*
		 *  USBアクティベートセットアップ
		 */
		sil_andw_mem((uint32_t *)(USBIEP_BASE(base, 0)+TOFF_USBEI_DIEPCTL), USB_OTG_DIEPCTL_MPSIZ);
		if((sil_rew_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DSTS)) & USB_OTG_DSTS_ENUMSPD) == DSTS_ENUMSPD_LS_PHY_6MHZ){
			sil_orw_mem((uint32_t *)(USBIEP_BASE(base, 0)+TOFF_USBEI_DIEPCTL), 3);
		}
		sil_orw_mem((uint32_t *)(USBD_BASE(base)+TOFF_USBD_DCTL), USB_OTG_DCTL_CGINAK);
		if(husb->devresetcallback != NULL)
			husb->devresetcallback(husb);
		clear_dev_int(base, USB_OTG_GINTSTS_ENUMDNE);
	}
    /*
	 *  RXQLEVEL割込み
	 */
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_RXFLVL) != 0){
		sil_andw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), USB_OTG_GINTSTS_RXFLVL);
		temp = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GRXSTSP));
		ep = &husb->OUT_ep[temp & USB_OTG_GRXSTSP_EPNUM];
		if(((temp & USB_OTG_GRXSTSP_PKTSTS) >> 17U) ==  STS_DATA_UPDT){
			if((temp & USB_OTG_GRXSTSP_BCNT) != 0){
				usbo_readPacket(base, ep->xfer_buff, (temp & USB_OTG_GRXSTSP_BCNT) >> 4U);
				ep->xfer_buff += (temp & USB_OTG_GRXSTSP_BCNT) >> 4U;
				ep->xfer_count += (temp & USB_OTG_GRXSTSP_BCNT) >> 4U;
			}
		}
		else if(((temp & USB_OTG_GRXSTSP_PKTSTS) >> 17U) ==  STS_SETUP_UPDT){
			usbo_readPacket(base, (uint8_t *)husb->Setup, 8U);
			ep->xfer_count += (temp & USB_OTG_GRXSTSP_BCNT) >> 4U;
		}
		sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK), USB_OTG_GINTSTS_RXFLVL);
	}
    /*
	 *  DEVICE-SOF割込み
	 */
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_SOF) != 0){
		if(husb->devsofcallback != NULL)
			husb->devsofcallback(husb);
		clear_dev_int(base, USB_OTG_GINTSTS_SOF);
	}
    /*
	 *  ISO IN未完結割込み
	 */
    if((get_int_status(base, msk) & USB_OTG_GINTSTS_IISOIXFR) != 0){
		if(husb->devisoincallback != NULL)
			husb->devisoincallback(husb, epnum);
		clear_dev_int(base, USB_OTG_GINTSTS_IISOIXFR);
    }
	/*
	 *  ISO OUT未完結割込み
	 */
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_PXFR_INCOMPISOOUT) != 0){
		if(husb->devisooutcallback != NULL)
			husb->devisooutcallback(husb, epnum);
		clear_dev_int(base, USB_OTG_GINTSTS_PXFR_INCOMPISOOUT);
	}
    /*
	 *  コネクションイベント割込み
	 */
    if((get_int_status(base, msk) & USB_OTG_GINTSTS_SRQINT) != 0){
		usbo_pcd_control(husb, 1);
		if(husb->devconnectcallback != NULL)
			husb->devconnectcallback(husb);
		clear_dev_int(base, USB_OTG_GINTSTS_SRQINT);
	}
    /*
	 *  ディスコネクションイベント割込み
	 */
    if((get_int_status(base, msk) & USB_OTG_GINTSTS_OTGINT) != 0){
		temp = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGINT));
		if((temp & USB_OTG_GOTGINT_SEDET) == USB_OTG_GOTGINT_SEDET){
			usbo_pcd_control(husb, 0);
			if(husb->devdisconnectcallback != NULL)
				husb->devdisconnectcallback(husb);
			if(husb->Init.usb_otg_mode == USB_OTG_MODE_DRD && husb->hostdisconnectcallback != NULL){
#if 1	/* ROI DEBUG */
				syslog_2(LOG_NOTICE, "usbo_pcd_irqhandler request host-discaonnect op_state(%d) lx_state(%d) !", husb->op_state, husb->lx_state);
#endif	/* ROI DEBUG */
				husb->hostdisconnectcallback(husb);
			}
		}
		sil_orw_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GOTGINT), temp);
	}
}

/*
 *  USB-OTG割込みサービスルーチン
 */
void
usb_otg_isr(intptr_t exinf)
{
	USB_OTG_Handle_t *husb = &Husb[INDEX_USB((uint32_t)exinf)];
	uint32_t base = husb->base;
	uint32_t msk  = sil_rew_mem((uint32_t *)(USBOTG_BASE(base)+TOFF_USBO_GINTMSK));
	if((get_int_status(base, msk) & USB_OTG_GINTSTS_CIDSCHG) != 0){
#if 1	/* ROI DEBUG */
		syslog_2(LOG_NOTICE, "## USB_OTG_GINTSTS_CIDSCHG op_state(%d) lx_state(%d) ##", husb->op_state, husb->lx_state);
#endif	/* ROI DEBUG */
		if(husb->op_state < OTG_STATE_A_HOST && husb->lx_state != DWC2_L0){
			if(get_usbmode(base) == USB_OTG_MODE_HOST && husb->otgidchangecallback != NULL){
				husb->op_state = OTG_STATE_A_HOST;
#if 1	/* ROI DEBUG */
				syslog_1(LOG_NOTICE, "## usb_otg_isr op_state(%d) ##", husb->op_state);
#endif	/* ROI DEBUG */
				husb->otgidchangecallback(husb);
			}
//			else
//				husb->op_state = OTG_STATE_A_PERIPHERAL;
//			husb->otgidchangecallback(husb);
		}
		clear_dev_int(base, USB_OTG_GINTSTS_CIDSCHG);
	}
	if(get_usbmode(base) == USB_OTG_MODE_DEVICE)
		usbo_pcd_irqhandler(husb);
	else
		usbo_hcd_irqhandler(husb);
}

